﻿using System.Reflection;
using Com.Ctrip.Framework.Apollo;
using Com.Ctrip.Framework.Apollo.Enums;
using Microsoft.Extensions.Configuration;

namespace BPA.Component.ApolloClient
{
    /// <summary>
    /// 配置基类
    /// </summary>
    public abstract class ApolloBaseConfig
    {
        /// <summary>
        /// 公共配置 配置对象,如果apollo初始化参数没有传入公共命名空间,该对象值为null
        /// </summary>
        protected readonly IConfig CommonConfig;

        /// <summary>
        /// 当前应用默认 配置
        /// </summary>
        protected readonly IConfig AppConfig;

        /// <summary>
        /// 微软配置对象
        /// </summary>
        protected readonly IConfiguration Configuration;

        /// <summary>
        /// apolloConfigurationManager
        /// </summary>
#pragma warning disable CS0618
        protected readonly ApolloConfigurationManager ApolloConfigurationManager;
#pragma warning restore CS0618

        /// <summary>
        /// autoWrites
        /// </summary>
        private readonly Dictionary<string, AutoWriteAttribute?> _autoWrites;

        /// <summary>
        /// 当前应用名称,全小写,同 AppName, Environment.GetEnvironmentVariable("APP_NAME")
        /// </summary>
        public static string APP_NAME { protected set; get; } = Environment.GetEnvironmentVariable("APP_NAME");

        /// <summary>
        /// 当前应用名称,全小写,默认 = Environment.GetEnvironmentVariable("APP_NAME")
        /// </summary>
        [AutoWrite(Key = "APP_NAME")]
        public string AppName
        {
            protected set => APP_NAME = value;
            get => APP_NAME;
        }

        /// <summary>
        /// 构造方法
        /// </summary>
        /// <param name="apolloConfigurationManager">apollo配置管理对象</param>
        /// <param name="configuration"></param>
#pragma warning disable CS0618
        public ApolloBaseConfig(ApolloConfigurationManager apolloConfigurationManager, IConfiguration configuration)
#pragma warning restore CS0618
        {
            _autoWrites = InitAutoWrite();
            ApolloConfigurationManager = apolloConfigurationManager;
            Configuration = configuration;

            //公共命名空间
            var apolloCommonNamespace = configuration.GetValue<string>("APOLLO_COMMON_NAMESPACE");
            if (!string.IsNullOrEmpty(apolloCommonNamespace))
            {
                CommonConfig = apolloConfigurationManager.GetConfig(apolloCommonNamespace).Result;
                CommonConfig.ConfigChanged += CommonConfig_ConfigChanged;
            }

            AppConfig = ApolloConfigurationManager.GetAppConfig().Result;
            AppConfig.ConfigChanged += AppConfig_ConfigChanged;

            //初始化配置值
            AutoWriteAll(_autoWrites);
        }

        /// <summary>
        /// 初始化AutoWrite
        /// </summary>
        private Dictionary<string, AutoWriteAttribute?> InitAutoWrite()
        {
            var autoWrites = new Dictionary<string, AutoWriteAttribute?>();
            var t = GetType();
            foreach (var p in t.GetProperties().OrderBy(it => it.Name).ToList())
            {
                var autoWrite = p.GetCustomAttribute<AutoWriteAttribute>();
                if (autoWrite == null)
                {
                    continue;
                }

                if (string.IsNullOrEmpty(autoWrite.Key))
                {
                    autoWrite.Key = p.Name;
                }

                if (autoWrites.ContainsKey(autoWrite.Key))
                {
                    throw new ArgumentException($"*apollo* auto write error: the key:{autoWrite.Key} is repetition!");
                }

                autoWrite.PropertyInfo = p;
                autoWrite.DefaultValue = p.GetValue(this);
                autoWrites.Add(autoWrite.Key, autoWrite);
            }

            return autoWrites;
        }

        /// <summary>
        /// 写入值
        /// </summary>
        /// <param name="key"></param>
        /// <param name="value"></param>
        /// <param name="useDefaultValue"></param>
        private void AutoWriteByKey(string key, string? value, bool useDefaultValue)
        {
            AutoWriteAttribute? autoWrite = null;
            try
            {
                if (!_autoWrites.ContainsKey(key))
                {
                    Console.WriteLine($"*apollo* auto write warning: the key[{key}] can not math property info,the value:{value}");
                    return;
                }

                autoWrite = _autoWrites[key];
                autoWrite?.SetValue(this, useDefaultValue ? null : value);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                Console.WriteLine(
                    $"*apollo* auto write error: key{key}," +
                    $"value:{value}," +
                    $"defaultValue:{autoWrite?.DefaultValue}," +
                    $"useDefaultValue:{useDefaultValue}");
            }
        }

        /// <summary>
        /// AutoWriteAll
        /// </summary>
        /// <param name="autoWrites"></param>
        private void AutoWriteAll(Dictionary<string, AutoWriteAttribute?> autoWrites)
        {
            string? defaultValue = null;
            foreach (var key in autoWrites.Keys)
            {
                var value = Configuration.GetValue<string>(key) ?? AppConfig?.GetProperty(key, defaultValue);
                if (value == null && CommonConfig != null)
                {
                    value = CommonConfig.GetProperty(key, defaultValue);
                }

                AutoWriteByKey(key, value, false);
            }
        }

        /// <summary>
        /// 公共配置触发改变
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void CommonConfig_ConfigChanged(IConfig sender, Com.Ctrip.Framework.Apollo.Model.ConfigChangeEventArgs args)
        {
            foreach (var key in args.ChangedKeys)
            {
                var configChange = args.GetChange(key);
                AutoWriteByKey(key, configChange.NewValue, configChange.ChangeType == PropertyChangeType.Deleted);
                OnApplicationConfigConfigChanged(key, configChange.NewValue, configChange.OldValue, configChange.ChangeType);
            }
        }

        /// <summary>
        /// 当前应用配置变化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void AppConfig_ConfigChanged(IConfig sender, Com.Ctrip.Framework.Apollo.Model.ConfigChangeEventArgs args)
        {
            foreach (var key in args.ChangedKeys)
            {
                var configChange = args.GetChange(key);
                AutoWriteByKey(key, configChange.NewValue, configChange.ChangeType == PropertyChangeType.Deleted);
                OnApplicationConfigConfigChanged(key, configChange.NewValue, configChange.OldValue, configChange.ChangeType);
            }
        }

        /// <summary>
        /// 当配置变化时触发
        /// </summary> 
        /// <param name="key">配置Key</param>
        /// <param name="newValue">配置新值</param>
        /// <param name="oldValue">配置旧值</param>
        /// <param name="changeType">配置变化类型</param> 
        protected virtual void OnApplicationConfigConfigChanged(string key, string? newValue, string? oldValue,
            PropertyChangeType changeType)
        {
            Console.WriteLine($"*apollo* auto write info: OnApplicationConfigConfigChanged:" +
                              $"{key}={{new:{newValue},old:{oldValue}}}," +
                              $"changeType:{changeType}");
        }
    }
}