﻿using Com.Ctrip.Framework.Apollo;
using Com.Ctrip.Framework.Apollo.Enums;
using Com.Ctrip.Framework.Apollo.Logging;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.ApolloClient
{
    /// <summary>
    /// apollo配置扩展类
    /// </summary>
    public static class ApolloConfigExtensions
    {
        private static IConfigurationRoot _configurationRoot;

        /// <summary>
        /// 创建 IConfiguration, 环境变量中需要的配置有
        /// <code>
        ///     APOLLO_META_SERVER_URL=         服务地址, 必填;
        ///     APOLLO_COMMON_NAMESPACE=        公共命名空间, 选填;
        ///     APP_NAME=                       应用名称,对应apollo项目名称, 必填;
        ///     APP_SERVER_IP=                  当前服务ip, 选填;
        ///     APP_SERVER_PORT=                当前服务端口, 选填;
        ///     ASPNETCORE_ENVIRONMENT=         生产[Production]，开发[Development];
        /// </code> 
        /// </summary>
        /// <param name="configuration">configuration</param>  
        /// <param name="namespaces">额外的命名空间</param>
        /// <exception cref="ArgumentNullException"></exception>
        /// <returns></returns>
        public static IConfigurationRoot AddApolloConfiguration(this IConfiguration configuration)
        {
            LogManager.UseConsoleLogging(LogLevel.Error);

            //apollo mete server 服务地址
            var apolloMetaServer = configuration.GetValue<string>("APOLLO_META_SERVER_URL") ??
                                   Environment.GetEnvironmentVariable("APOLLO_META_SERVER_URL");
            Console.WriteLine($"*apollo* build configuration info: [APOLLO_META_SERVER_URL]={apolloMetaServer}");

            if (string.IsNullOrEmpty(apolloMetaServer))
            {
#pragma warning disable CA2208
                throw new ArgumentNullException(
                    @$"*apollo* build configuration error: the [APOLLO_META_SERVER_URL] is empty!");
#pragma warning restore CA2208
            }

            var environment = Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT");
            Console.WriteLine($"*apollo* build configuration info: [ASPNETCORE_ENVIRONMENT]={environment}");

            //公共配置命名空间
            var apolloCommonNamespace = configuration.GetValue<string>("APOLLO_COMMON_NAMESPACE") ??
                                        Environment.GetEnvironmentVariable("APOLLO_COMMON_NAMESPACE");
            

            //app name
            var appName = configuration.GetValue<string>("APP_NAME") ?? Environment.GetEnvironmentVariable("APP_NAME");
            Console.WriteLine($"*apollo* build configuration info: [APP_NAME]={appName}");

            if (string.IsNullOrEmpty(appName))
            {
#pragma warning disable CA2208
                throw new ArgumentNullException($@"*apollo*  build configuration error: the [APP_NAME] is empty!");
#pragma warning restore CA2208
            }

            // //当前服务ip
            // var appServerIp = configuration.GetValue<string>("APP_SERVER_IP") ??
            //                   Environment.GetEnvironmentVariable("APP_SERVER_IP");
            // Console.WriteLine($"*apollo* build configuration info: [APP_SERVER_IP]={appServerIp}");
            //
            // //当前服务端口
            // var appServerPort = configuration.GetValue<string>("APP_SERVER_PORT") ??
            //                     Environment.GetEnvironmentVariable("APP_SERVER_PORT");
            // Console.WriteLine($"*apollo* build configuration info: [APP_SERVER_PORT]={appServerPort}");

            //localIp
            // var localIp = string.IsNullOrEmpty(appServerIp) || string.IsNullOrEmpty(appServerPort)
            //     ? null
            //     : $"{appServerIp}:{appServerPort}";

            var builder = new ConfigurationBuilder();
            builder.AddConfiguration(configuration);

            var env = environment switch
            {
                "Development" => Env.Dev,
                "Testing" => Env.Uat,
                "Production" => Env.Pro,
                _ => throw new Exception("ASPNETCORE_ENVIRONMENT ERROR")
            };

            var apollo = builder.AddApollo(new ApolloOptions
            {
                AppId = appName,
                MetaServer = apolloMetaServer,
                Env = env,
            });

            apollo.AddDefault();

            if (!string.IsNullOrEmpty(apolloCommonNamespace))
            {
                if (env == Env.Uat)
                    apolloCommonNamespace = "Test.Common"; // 测试环境写死
                apollo.AddNamespace(apolloCommonNamespace);
            }
            
            Console.WriteLine($"*apollo* build configuration info: [APOLLO_COMMON_NAMESPACE]={apolloCommonNamespace}");

            _configurationRoot = builder.Build();
            return _configurationRoot;
        }

        /// <summary>
        /// UseApollo
        /// </summary>
        /// <typeparam name="TApolloBaseConfiguration"></typeparam>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddApollo<TApolloBaseConfiguration>(this IServiceCollection services)
            where TApolloBaseConfiguration : ApolloBaseConfig
        {
            services.AddSingleton<IConfiguration>(_configurationRoot ?? throw new InvalidOperationException());
            services.AddSingleton(_configurationRoot);
#pragma warning disable CS0618
            services.AddSingleton<ApolloConfigurationManager>();
#pragma warning restore CS0618
            services.AddSingleton<TApolloBaseConfiguration>();
            return services;
        }
    }
}