﻿using System.Data;
using System.Data.Common;
using BPA.Component.ClickHouseClient.Models;
using ClickHouse.Client.ADO;
using ClickHouse.Client.Copy;
using Dapper;

namespace BPA.Component.ClickHouseClient
{
    /// <summary>
    /// 连接扩展
    /// </summary>
    public static class ClickHouseConnectionExtensions
    {
        /// <summary>
        /// 批量插入
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="tableName"></param>
        /// <param name="columns"></param>
        /// <param name="dataRows"></param>
        public static void BulkCopy(this ClickHouseConnection connection, string tableName, IReadOnlyCollection<string> columns,
            IEnumerable<object[]> dataRows)
        {
            using var bulkCopy = new ClickHouseBulkCopy(connection)
            {
                DestinationTableName = tableName,
                BatchSize = dataRows.Count()
            };
            bulkCopy.WriteToServerAsync(dataRows, columns).GetAwaiter().GetResult();
        }

        /// <summary>
        /// 批量插入
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="tableName"></param>
        /// <param name="columns"></param>
        /// <param name="dataRows"></param>
        public static void BulkCopy(this ClickHouseConnection connection, string tableName, IReadOnlyCollection<string> columns,
            IEnumerable<Dictionary<string, object>> dataRows)
        {
            using var bulkCopy = new ClickHouseBulkCopy(connection)
            {
                DestinationTableName = tableName,
                BatchSize = dataRows.Count()
            };
            var insertRows = new List<object[]>();
            foreach (var item in dataRows)
            {
                var datas = columns.Select(x => item[x]).ToArray();
                insertRows.Add(datas);
            }

            bulkCopy.WriteToServerAsync(insertRows, columns).GetAwaiter().GetResult();
        }

        /// <summary>
        /// 批量插入
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="tableName"></param>
        /// <param name="columns"></param>
        /// <param name="dataRows"></param>
        /// <param name="proccessColumnsValue">处理列对应的值,    </param>
        public static void BulkCopy(
            this ClickHouseConnection connection,
            string tableName, IReadOnlyCollection<string> columns,
            IEnumerable<Dictionary<string, object>> dataRows,
            Func<string, object, object> proccessColumnsValue)
        {
            using var bulkCopy = new ClickHouseBulkCopy(connection)
            {
                DestinationTableName = tableName,
                BatchSize = dataRows.Count()
            };
            var insertRows = new List<object[]>();
            foreach (var item in dataRows)
            {
                var datas = columns.Select(x => proccessColumnsValue(x, item[x])).ToArray();
                insertRows.Add(datas);
            }

            bulkCopy.WriteToServerAsync(insertRows, columns).GetAwaiter().GetResult();
        }

        /// <summary>
        /// 批量插入
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="tableName"></param>
        /// <param name="columns"></param>
        /// <param name="dataRows"></param> 
        public static void BulkCopy(
            this ClickHouseConnection connection,
            string tableName,
            List<BulkCopySourceTargetColumnInfo> columns,
            IEnumerable<Dictionary<string, object>> dataRows)
        {
            using var bulkCopy = new ClickHouseBulkCopy(connection)
            {
                DestinationTableName = tableName,
                BatchSize = dataRows.Count()
            };
            var insertRows = new List<object[]>();
            foreach (var sourceItem in dataRows)
            {
                var insertDataRow = columns.Select(column =>
                {
                    sourceItem.TryGetValue(column.SourceColum, out object vObj);
                    if (column.ProccessValue != null)
                        vObj = column.ProccessValue.Invoke(vObj);
                    if (vObj == null)
                        vObj = column.DefaultValue;
                    return vObj;
                }).ToArray();
                insertRows.Add(insertDataRow);
            }

            bulkCopy.WriteToServerAsync(insertRows, columns.Select(x => x.TargetColum).ToList()).GetAwaiter().GetResult();
        }

        /// <summary>
        /// Query， 修正时区
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="connection"></param>
        /// <param name="sql"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static IEnumerable<T> Query<T>(this IDbConnection connection, string sql, Dictionary<string, object> param = null)
        {
            var result = connection.QueryAsync<T>(sql, param).GetAwaiter().GetResult();
            if (result == null || result.Count() == 0)
                return result;
            var dateTimeType = typeof(DateTime);
            var dateTimeTypeNull = typeof(DateTime?);
            var dateTimeProperties = typeof(T).GetProperties()
                .Where(x => x.PropertyType == dateTimeType || x.PropertyType == dateTimeTypeNull).ToList();
            if (dateTimeProperties.Count == 0)
                return result;
            foreach (var item in result)
            {
                foreach (var property in dateTimeProperties)
                {
                    var value = property.GetValue(item);
                    if (value == null)
                        continue;
                    if (value is DateTime dateTime)
                    {
                        if (dateTime == ClickHouseConts.MinDateTime)
                            property.SetValue(item, ClickHouseConts.MinDateTimeLocal);
                        else
                            property.SetValue(item, dateTime.ToLocalTime());
                    }
                    else
                    {
                        var dateTimeNull = value as DateTime?;
                        if (dateTimeNull == null)
                            continue;
                        else if (dateTimeNull == ClickHouseConts.MinDateTime)
                            property.SetValue(item, ClickHouseConts.MinDateTimeLocal);
                        else
                            property.SetValue(item, dateTimeNull?.ToLocalTime());
                    }
                }
            }

            return result;
        }

        /// <summary>
        /// 执行sql并返回单行单列值
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="sql"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static async Task<T> ExecuteScalarAsync<T>(this ClickHouseConnection connection, string sql,
            Dictionary<string, object> param = null)
        {
            using var command = connection.CreateCommand();
            command.CommandText = sql;
            if (param != null && param.Count > 0)
            {
                foreach (var kv in param)
                {
                    var parameter = command.CreateParameter();
                    parameter.ParameterName = kv.Key;
                    parameter.Value = kv.Value;
                    command.Parameters.Add(parameter);
                }
            }

            return (T) await command.ExecuteScalarAsync().ConfigureAwait(false);
        }


        /// <summary>
        /// 执行sql并返回单行单列值
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="sql"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static async Task<object> ExecuteScalarAsync(this ClickHouseConnection connection, string sql,
            Dictionary<string, object> param = null)
        {
            using var command = connection.CreateCommand();
            command.CommandText = sql;
            if (param != null && param.Count > 0)
            {
                foreach (var kv in param)
                {
                    var parameter = command.CreateParameter();
                    parameter.ParameterName = kv.Key;
                    parameter.Value = kv.Value;
                    command.Parameters.Add(parameter);
                }
            }

            return await command.ExecuteScalarAsync().ConfigureAwait(false);
        }


        /// <summary>
        /// 执行sql并返回单行单列值
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="sql"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static T ExecuteScalar<T>(this ClickHouseConnection connection, string sql, Dictionary<string, object> param = null)
        {
            using var command = connection.CreateCommand();
            command.CommandText = sql;
            if (param != null && param.Count > 0)
            {
                foreach (var kv in param)
                {
                    var parameter = command.CreateParameter();
                    parameter.ParameterName = kv.Key;
                    parameter.Value = kv.Value;
                    command.Parameters.Add(parameter);
                }
            }

            return (T) command.ExecuteScalarAsync().ConfigureAwait(false).GetAwaiter().GetResult();
        }

        /// <summary>
        /// 执行sql 并返回datereader
        /// </summary>
        /// <param name="connection"></param>
        /// <param name="sql"></param>
        /// <param name="param"></param>
        /// <returns></returns>
        public static async Task<DbDataReader> ExecuteReaderAsync(this ClickHouseConnection connection, string sql,
            Dictionary<string, object> param = null)
        {
            using var command = connection.CreateCommand();
            command.CommandText = sql;
            if (param != null && param.Count > 0)
            {
                foreach (var kv in param)
                {
                    var parameter = command.CreateParameter();
                    parameter.ParameterName = kv.Key;
                    parameter.Value = kv.Value;
                    command.Parameters.Add(parameter);
                }
            }

            return await command.ExecuteReaderAsync().ConfigureAwait(false);
        }
    }
}