﻿using ClickHouse.Client.ADO;

namespace BPA.Component.ClickHouseClient
{
    /// <summary>
    /// ClickHouseProvider
    /// </summary>
    public abstract class ClickHouseProvider : IDisposable
    {
        /// <summary>
        /// 企业和别名的关系
        /// </summary>
        protected readonly Dictionary<long, ClickHouseConnection> corpWithmongoDbContexts = new Dictionary<long, ClickHouseConnection>();

        /// <summary>
        /// 别名和ClickHouse实例的对应关系
        /// </summary>
        protected readonly Dictionary<string, ClickHouseConnection> aliasWithMongoDbContexts =
            new Dictionary<string, ClickHouseConnection>();

        /// <summary>
        /// 企业id和数据库别名对应关系
        /// </summary>
        protected readonly Dictionary<long, string> corpWithAlias = new Dictionary<long, string>();

        /// <summary>
        /// 别名和连接配置关系
        /// </summary>
        public IReadOnlyDictionary<string, ClickHouseConfig> AliasConfig { private set; get; }

        /// <summary>
        /// MongonDbFactory
        /// </summary>
        /// <param name="configs"></param>
        public ClickHouseProvider(List<ClickHouseConfig> configs)
        {
            SetConfigs(configs);
        }

        /// <summary>
        /// 设置数据库连接配置
        /// </summary>
        /// <param name="configs"></param>
        public virtual void SetConfigs(List<ClickHouseConfig> configs)
        {
            lock (this)
            {
                var dic = new Dictionary<string, ClickHouseConfig>();
                foreach (var config in configs)
                {
                    if (dic.ContainsKey(config.Alias))
                        throw new ArgumentException($"数据库{this.GetType().Name}的配置别名[{config.Alias}]出现重复");
                    dic.Add(config.Alias, config);
                }

                AliasConfig = dic;
                corpWithmongoDbContexts.Clear();
                aliasWithMongoDbContexts.Clear();
                corpWithAlias.Clear();
            }
        }

        /// <summary>
        /// 根据企业ID获取ClickHouse连接
        /// </summary>
        /// <param name="corpId">企业ID</param>
        /// <returns></returns>
        public virtual ClickHouseConnection GetConnection(long corpId)
        {
            if (corpWithmongoDbContexts.ContainsKey(corpId))
                return corpWithmongoDbContexts[corpId];
            lock (corpWithmongoDbContexts)
            {
                if (corpWithmongoDbContexts.ContainsKey(corpId))
                    return corpWithmongoDbContexts[corpId];
                var dbAlias = corpWithAlias.ContainsKey(corpId) ? corpWithAlias[corpId] : GetConnectionAliasByCropId(corpId);
                if (dbAlias == null)
                    throw new ArgumentException($"无法获取到企业[{corpId}]的{this.GetType().Name}数据库别名!");
                if (!AliasConfig.ContainsKey(dbAlias))
                    throw new ArgumentException($"无法获取到企业[{corpId}]的{this.GetType().Name}数据库别名[{dbAlias}]对应的数据库连接!");
                //缓存别名
                corpWithAlias[corpId] = dbAlias;
                //从别名和链接对应关系中获取
                if (aliasWithMongoDbContexts.ContainsKey(dbAlias))
                {
                    return corpWithmongoDbContexts[corpId] = aliasWithMongoDbContexts[dbAlias];
                }

                var connection = GetConnection(dbAlias);
                corpWithmongoDbContexts[corpId] = connection;
                aliasWithMongoDbContexts[dbAlias] = connection;
                return connection;
            }
        }

        /// <summary>
        /// 根据别名获取ClickHouse连接
        /// </summary>
        /// <param name="dbAlias"></param>
        /// <returns></returns>
        public ClickHouseConnection GetConnection(string dbAlias)
        {
            ClickHouseConnection connection = null;
            lock (aliasWithMongoDbContexts)
            {
                if (aliasWithMongoDbContexts.ContainsKey(dbAlias))
                    connection = aliasWithMongoDbContexts[dbAlias];
                else
                {
                    var config = AliasConfig[dbAlias];
                    connection = new ClickHouseConnection(config.ConnectionString);
                    aliasWithMongoDbContexts[dbAlias] = connection;
                }
            }

            return connection;
        }

        /// <summary>
        /// 获取所有的数据库操作对象  别名+数据库操作对象
        /// </summary>
        /// <returns></returns>
        public IReadOnlyDictionary<string, ClickHouseConnection> GetConnections()
        {
            if (AliasConfig.Count != aliasWithMongoDbContexts.Count)
            {
                foreach (var dbAlias in AliasConfig.Keys)
                {
                    GetConnection(dbAlias);
                }
            }

            return aliasWithMongoDbContexts;
        }

        /// <summary>
        /// 根据企业Id获取数据库别名,子类必须实现
        /// </summary>
        /// <returns></returns>
        protected abstract string GetConnectionAliasByCropId(long corpId);

        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            foreach (var kv in corpWithmongoDbContexts)
            {
                kv.Value.Dispose();
            }
        }

        /// <summary>
        /// 获取企业的连接字符串别名
        /// </summary>
        public string GetCropAlias(long corpId)
        {
            lock (corpWithAlias)
            {
                var dbAlias = corpWithAlias.ContainsKey(corpId) ? corpWithAlias[corpId] : GetConnectionAliasByCropId(corpId);
                if (dbAlias == null) return null;
                //缓存别名
                corpWithAlias[corpId] = dbAlias;
                return dbAlias;
            }
        }
    }
}