﻿using BPA.Component.DTOCommon.Langs;
using BPA.Component.Extensions;
using Microsoft.AspNetCore.Http;
using Newtonsoft.Json;

namespace BPA.Component.DTOCommon.BaseDTOs
{
    /// <summary>
    /// BaseRsp
    /// </summary>
    public class BaseResult
    {
        /// <summary>
        /// lang
        /// </summary>
        [JsonProperty]
        internal Lang Lang { set; get; }

        /// <summary>
        /// 更多的中文提示,返回给接口用方
        /// </summary>
        [JsonProperty]
        internal string MoreZhMsg { set; get; }

        /// <summary>
        /// 更多的英文提示,返回给接口用方
        /// </summary>
        [JsonProperty]
        internal string MoreEnMsg { set; get; }

        /// <summary>
        /// 返回代码
        /// </summary>
        public string Code { set; get; }

        /// <summary>
        /// 返回消息
        /// </summary>
        public string Msg { get; set; }

        /// <summary>
        /// 是否成功 Code == OK
        /// </summary>
        public bool IsOK
        {
            get { return Code == ConstLangPackge.OK.Key; }
        }

        /// <summary>
        /// 跟踪
        /// </summary>
        public string TrackId { set; get; }

        /// <summary>
        /// 耗时
        /// </summary>
        public string Duration { get; set; }

        /// <summary>
        /// 构造方法
        /// </summary>
        public BaseResult()
        {
        }

        /// <summary>
        /// BaseResult
        /// </summary>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <param name="httpContext"></param>
        public BaseResult(Lang lang, string moreZh = null, string moreEn = null)
        {
            DoBuild(lang, moreZh, moreEn);
        }

        /// <summary>
        /// 获取当前返回的语言包
        /// </summary>
        /// <returns></returns>
        public Lang GetLang() => this.Lang;

        /// <summary>
        /// 异常渲染返回结果中的 core 和 msg 属性值
        /// </summary>
        /// <param name="langType"></param>
        /// <param name="httpRequestMessage"></param>
        /// <returns></returns>
        public BaseResult Render(LangType langType = LangType.Zn, HttpRequestMessage httpRequestMessage = null)
        {
            // ReSharper disable once ConditionIsAlwaysTrueOrFalse
            if (Lang == null)
                return this;
            
            Code = Lang.Key;
            Msg = Lang[langType];
            if (langType == LangType.Zn && !string.IsNullOrEmpty(MoreZhMsg))
                Msg += $":{MoreZhMsg}";
            else if (langType == LangType.En && !string.IsNullOrEmpty(MoreEnMsg))
                Msg += $":{MoreEnMsg}";

            // if (httpRequestMessage.Headers.Contains("X-TraceId"))
            // {
            //     try
            //     {
            //         TrackId = httpRequestMessage.Headers.GetValues("X-TraceId").ToList()[0];
            //     }
            //     catch (Exception e)
            //     {
            //         Console.WriteLine(e);
            //     }
            // }

            return this;
        }

        /// <summary>
        /// 成功
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public BaseResult OK(string moreZh = null, string moreEn = null) => DoBuild(ConstLangPackge.OK, moreZh, moreEn);

        /// <summary>
        /// 失败
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public BaseResult Fail(string moreZh = null, string moreEn = null) =>
            DoBuild(ConstLangPackge.Fail, moreZh, moreEn);

        /// <summary>
        /// set相关属性
        /// </summary>
        /// <param name="lang"></param>
        /// <param name="moreZh"></param>
        /// <param name="moreEn"></param>
        /// <returns></returns>
        protected BaseResult DoBuild(Lang lang, string moreZh = null, string moreEn = null)
        {
            if (lang == null)
                throw new ArgumentNullException(nameof(lang));
            this.Code = lang.Key;
            this.Lang = lang;
            this.MoreZhMsg = moreZh;
            this.MoreEnMsg = moreEn;
            return this;
        }

        /// <summary>
        /// Build
        /// </summary>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResult Build(Lang lang, string moreZh = null, string moreEn = null) =>
            new BaseResult(lang, moreZh, moreEn);

        /// <summary>
        /// BuildOK
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResult BuildOK(string moreZh = null, string moreEn = null) =>
            new BaseResult(ConstLangPackge.OK, moreZh, moreEn);

        /// <summary>
        /// BuildFail
        /// </summary>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static BaseResult BuildFail(string moreZh = null, string moreEn = null) =>
            new BaseResult(ConstLangPackge.Fail, moreZh, moreEn);

        /// <summary>
        /// Build
        /// </summary>
        /// <typeparam name="ResultDTO"></typeparam>
        /// <param name="lang">语言配置</param>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static ResultDTO Build<ResultDTO>(Lang lang, string moreZh = null, string moreEn = null)
            where ResultDTO : BaseResult, new() => new ResultDTO {Lang = lang, MoreZhMsg = moreZh, MoreEnMsg = moreEn};

        /// <summary>
        /// BuildFail
        /// </summary>
        /// <typeparam name="ResultDTO"></typeparam>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static ResultDTO BuildFail<ResultDTO>(string moreZh = null, string moreEn = null)
            where ResultDTO : BaseResult, new() => new ResultDTO
            {Lang = ConstLangPackge.Fail, MoreZhMsg = moreZh, MoreEnMsg = moreEn};

        /// <summary>
        /// BuildOK
        /// </summary>
        /// <typeparam name="ResultDTO"></typeparam>
        /// <param name="moreZh">更多的中文提示,返回给接口用方</param>
        /// <param name="moreEn">更多的英文提示,返回给接口用方</param>
        /// <returns></returns>
        public static ResultDTO BuildOK<ResultDTO>(string moreZh = null, string moreEn = null)
            where ResultDTO : BaseResult, new() => new ResultDTO
            {Lang = ConstLangPackge.OK, MoreZhMsg = moreZh, MoreEnMsg = moreEn};

        /// <summary>
        /// 复制属性到目标对象属性
        /// </summary>
        /// <typeparam name="TCopyTo"></typeparam>
        /// <param name="targer"></param>
        /// <returns></returns>
        public TCopyTo CopyTo<TCopyTo>(TCopyTo targer) where TCopyTo : BaseResult
        {
            targer.Lang = this.Lang;
            targer.MoreZhMsg = this.MoreZhMsg;
            targer.MoreEnMsg = this.MoreEnMsg;
            targer.Msg = this.Msg;
            return targer;
        }

        /// <summary>
        /// 复制属性到目标对象属性,自动创建目标对象
        /// </summary>
        /// <typeparam name="TCopyTo"></typeparam> 
        /// <returns></returns>
        public TCopyTo CopyTo<TCopyTo>() where TCopyTo : BaseResult, new()
        {
            var targer = new TCopyTo
            {
                Lang = this.Lang,
                MoreZhMsg = this.MoreZhMsg,
                MoreEnMsg = this.MoreEnMsg,
                Msg = this.Msg
            };
            return targer;
        }
    }
}