﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using SqlSugar;

namespace BPA.Component.DbClient
{
    /// <summary>
    /// 数据库客户端基类
    /// </summary>
    public abstract class BaseDbClient<TDbClient> : SqlSugarClient
        where TDbClient : BaseDbClient<TDbClient>
    {
        /// <summary>
        /// Logger
        /// </summary>
        private readonly ILogger<TDbClient> _logger;

        /// <summary>
        /// 是否打印SQL
        /// </summary>
        private readonly bool _isLogSql;

        /// <summary>
        /// BaseDbClient
        /// </summary>
        /// <param name="config"></param>
        /// <param name="isLogSql"></param>
        /// <param name="logger"></param>
        protected BaseDbClient(ConnectionConfig config, bool isLogSql = true, ILogger<TDbClient> logger = null) :
            base(config)
        {
            _logger = logger;
            _isLogSql = isLogSql;
            ConfigAop();
        }

        /// <summary>
        /// 创建数据库配置对象
        /// </summary>
        /// <param name="dbConfig">数据库连接配置</param>  
        /// <param name="dbType">数据库类型,默认MySql</param> 
        /// <returns></returns>
        protected static ConnectionConfig BuildConnectionConfig(DbConfig dbConfig, DbType dbType = DbType.MySql)
        {
            if (dbConfig == null)
                throw new ArgumentNullException(nameof(dbConfig));

            var connectionConfig = new ConnectionConfig
            {
                ConnectionString = dbConfig.MasterDbConnectionString,
                DbType = dbType,
                IsAutoCloseConnection = true,
                SlaveConnectionConfigs = new List<SlaveConnectionConfig>()
            };

            if (dbConfig.SlaveDbConnectionConfigs is not {Length: > 0})
                return connectionConfig;

            foreach (var dbReader in dbConfig.SlaveDbConnectionConfigs)
            {
                var slaveConnectionConfig = new SlaveConnectionConfig()
                {
                    ConnectionString = dbReader
                };
                connectionConfig.SlaveConnectionConfigs.Add(slaveConnectionConfig);
            }

            return connectionConfig;
        }

        /// <summary>
        /// 配置Aop
        /// </summary>
        private void ConfigAop()
        {
            Aop.OnError = ex =>
            {
                _logger?.LogError(ex,
                    @"执行SQL异常:" +
                    $"\nSQL:{ex.Sql}  " +
                    $"\n参数:" +
                    $"\n {JsonConvert.SerializeObject(ex.Parametres, Formatting.Indented)} " +
                    $"\nSource: {ex.Source}");
            };

            Aop.OnLogExecuted = (sql, pars) =>
            {
                if (!_isLogSql) return;

                var log =
                    $"\nSQL : \n{sql}" +
                    $"\n参数: \n{JsonConvert.SerializeObject(pars.ToDictionary(it => it.ParameterName, it => it.Value?.ToString()), Formatting.Indented)}" +
                    $"\n耗时: \n{Ado.SqlExecutionTime.TotalMilliseconds}ms";

                _logger?.LogInformation(log);
            };
        }
    }
}