﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Threading.Tasks;
using SqlSugar;

namespace BPA.Component.DbClient.RepositoryModel
{
    /// <summary>
    /// 基础仓储
    /// 所有数据表仓储都需要继承该基础仓储
    /// </summary>
    /// <typeparam name="TDbClient">DB客户端</typeparam>
    /// <typeparam name="TEntity">实体类</typeparam>
    public class BaseDbRepository<TDbClient, TEntity> : IDisposable, IDbRepository<TEntity>
        where TDbClient : BaseDbClient<TDbClient>
        where TEntity : class,
        new()
    {
        /// <summary>
        /// _simpleClient
        /// </summary>
        private SimpleClient<TEntity> _simpleClient;

        /// <summary>
        /// SimpleClient
        /// </summary>
        public SimpleClient<TEntity> SimpleClient => _simpleClient ??= DbClient.GetSimpleClient<TEntity>();

        /// <summary>
        /// DbClient
        /// </summary>
        SqlSugarClient IDbRepository<TEntity>.DbClient => DbClient;

        /// <summary>
        /// 当前BaseRepository中使用的数据库
        /// </summary>
        public BaseDbClient<TDbClient> DbClient { internal set; get; }

        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="dbClient">dbConfig</param>
        public BaseDbRepository(BaseDbClient<TDbClient> dbClient) => DbClient = dbClient;

        /// <summary>
        /// 构造函数
        /// </summary> 
        internal BaseDbRepository()
        {
        }

        /// <summary>
        /// 销毁
        /// </summary>
        public void Dispose() => DbClient?.Dispose();


        /// <summary>
        /// InsertAsync
        /// </summary>
        /// <param name="insertObj"></param>
        /// <returns></returns>
        public async Task<bool> InsertAsync(TEntity insertObj) =>
            await DbClient.Insertable(insertObj).ExecuteCommandAsync() > 0;

        /// <summary>
        /// InsertIdIntoEntityAsync
        /// </summary>
        /// <param name="insertObj"></param>
        /// <returns></returns>
        public Task<bool> InsertIdIntoEntityAsync(TEntity insertObj) =>
            DbClient.Insertable(insertObj).ExecuteCommandIdentityIntoEntityAsync();

        /// <summary>
        /// InsertRangeAsync
        /// </summary>
        /// <param name="insertObjs"></param>
        /// <returns></returns>
        public Task<int> InsertRangeAsync(TEntity[] insertObjs) =>
            DbClient.Insertable(insertObjs).ExecuteCommandAsync();

        /// <summary>
        /// InsertRangeAsync
        /// </summary>
        /// <param name="insertObjs"></param>
        /// <returns></returns>
        public Task<int> InsertRangeAsync(List<TEntity> insertObjs) =>
            DbClient.Insertable(insertObjs).ExecuteCommandAsync();

        /// <summary>
        /// UpdateAsync
        /// </summary>
        /// <param name="updateObj"></param>
        /// <returns></returns>
        public Task<bool> UpdateAsync(TEntity updateObj) => SimpleClient.UpdateAsync(updateObj);

        /// <summary>
        /// UpdateAsync
        /// </summary>
        /// <param name="columns"></param>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<bool> Update(Expression<Func<TEntity, TEntity>> columns,
            Expression<Func<TEntity, bool>> whereExpression) =>
            SimpleClient.UpdateAsync(columns, whereExpression);

        /// <summary>
        /// UpdateRangeAsync
        /// </summary>
        /// <param name="updateObjs"></param>
        /// <returns></returns>
        public Task<bool> UpdateRangeAsync(TEntity[] updateObjs) => SimpleClient.UpdateRangeAsync(updateObjs);

        /// <summary>
        /// UpdateRange
        /// </summary>
        /// <param name="updateObjs"></param>
        /// <returns></returns>
        public Task<bool> UpdateRangeAsync(List<TEntity> updateObjs) => SimpleClient.UpdateRangeAsync(updateObjs);

        /// <summary>
        /// DeleteAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<bool> DeleteAsync(Expression<Func<TEntity, bool>> whereExpression) =>
            SimpleClient.DeleteAsync(whereExpression);

        /// <summary>
        /// DeleteByIdAsync
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public bool DeleteByIdAsync(dynamic id) => SimpleClient.DeleteByIdAsync(id);

        /// <summary>
        /// CountAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<int> CountAsync(Expression<Func<TEntity, bool>> whereExpression) =>
            SimpleClient.CountAsync(whereExpression);

        /// <summary>
        /// IsAnyAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<bool> IsAnyAsync(Expression<Func<TEntity, bool>> whereExpression) =>
            SimpleClient.IsAnyAsync(whereExpression);

        /// <summary>
        /// GetByIdAsync
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public Task<TEntity> GetByIdAsync(dynamic id) => SimpleClient.GetByIdAsync(id);

        /// <summary>
        /// FirstAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <param name="selectExpression"></param>
        /// <returns></returns>
        public Task<TResult> FirstAsync<TResult>(Expression<Func<TEntity, bool>> whereExpression,
            Expression<Func<TEntity, TResult>> selectExpression) => DbClient
            .Queryable<TEntity>()
            .Where(whereExpression)
            .Select(selectExpression)
            .FirstAsync();

        /// <summary>
        /// FirstAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<TEntity> FirstAsync(Expression<Func<TEntity, bool>> whereExpression) => DbClient
            .Queryable<TEntity>()
            .FirstAsync(whereExpression);


        /// <summary>
        /// GetListAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        public Task<List<TEntity>> GetListAsync(Expression<Func<TEntity, bool>> whereExpression) =>
            SimpleClient.GetListAsync(whereExpression);

        /// <summary>
        /// 查询
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="whereExpression"></param>
        /// <param name="expression"></param>
        /// <returns></returns>
        public Task<List<TResult>> GetListAsync<TResult>(Expression<Func<TEntity, bool>> whereExpression,
            Expression<Func<TEntity, TResult>> expression) => DbClient
            .Queryable<TEntity>()
            .Where(whereExpression)
            .Select(expression)
            .ToListAsync();
    }
}