﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Threading.Tasks;
using SqlSugar;

namespace BPA.Component.DbClient.RepositoryModel
{
    /// <summary>
    /// 基础仓储
    /// 所有数据表仓储都需要继承该基础仓储
    /// </summary>
    /// <typeparam name="TEntity">实体类</typeparam>
    public interface IDbRepository<TEntity> : IDisposable where TEntity : class, new()
    {
        /// <summary>
        /// 数据库操作对象
        /// </summary>
        SqlSugarClient DbClient { get; }

        /// <summary>
        /// 当前实体数据库操作对象
        /// </summary>
        public SimpleClient<TEntity> SimpleClient { get; }

        /// <summary>
        /// InsertAsync
        /// </summary>
        /// <param name="insertObj"></param>
        /// <returns></returns>
        Task<bool> InsertAsync(TEntity insertObj);

        /// <summary>
        /// InsertIdIntoEntityAsync
        /// </summary>
        /// <param name="insertObj"></param>
        /// <returns></returns>
        Task<bool> InsertIdIntoEntityAsync(TEntity insertObj);

        /// <summary>
        /// InsertRangeAsync
        /// </summary>
        /// <param name="insertObjs"></param>
        /// <returns></returns>
        Task<int> InsertRangeAsync(TEntity[] insertObjs);

        /// <summary>
        /// InsertRangeAsync
        /// </summary>
        /// <param name="insertObjs"></param>
        /// <returns></returns>
        Task<int> InsertRangeAsync(List<TEntity> insertObjs);

        /// <summary>
        /// UpdateAsync
        /// </summary>
        /// <param name="updateObj"></param>
        /// <returns></returns>
        Task<bool> UpdateAsync(TEntity updateObj);

        /// <summary>
        /// UpdateAsync
        /// </summary>
        /// <param name="columns"></param>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<bool> Update(Expression<Func<TEntity, TEntity>> columns,
            Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// UpdateRangeAsync
        /// </summary>
        /// <param name="updateObjs"></param>
        /// <returns></returns>
        Task<bool> UpdateRangeAsync(TEntity[] updateObjs);

        /// <summary>
        /// UpdateRange
        /// </summary>
        /// <param name="updateObjs"></param>
        /// <returns></returns>
        Task<bool> UpdateRangeAsync(List<TEntity> updateObjs);

        /// <summary>
        /// DeleteAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<bool> DeleteAsync(Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// DeleteByIdAsync
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        bool DeleteByIdAsync(dynamic id);

        /// <summary>
        /// CountAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<int> CountAsync(Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// IsAnyAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<bool> IsAnyAsync(Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// GetByIdAsync
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<TEntity> GetByIdAsync(dynamic id);

        /// <summary>
        /// FirstAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <param name="selectExpression"></param>
        /// <returns></returns>
        Task<TResult> FirstAsync<TResult>(Expression<Func<TEntity, bool>> whereExpression,
            Expression<Func<TEntity, TResult>> selectExpression);

        /// <summary>
        /// FirstAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<TEntity> FirstAsync(Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// GetListAsync
        /// </summary>
        /// <param name="whereExpression"></param>
        /// <returns></returns>
        Task<List<TEntity>> GetListAsync(Expression<Func<TEntity, bool>> whereExpression);

        /// <summary>
        /// 查询
        /// </summary>
        /// <typeparam name="TResult"></typeparam>
        /// <param name="whereExpression"></param>
        /// <param name="expression"></param>
        /// <returns></returns>
        Task<List<TResult>> GetListAsync<TResult>(Expression<Func<TEntity, bool>> whereExpression,
            Expression<Func<TEntity, TResult>> expression); 
    }
}