﻿using System.Globalization;
using System.Runtime.CompilerServices;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// 分布式ID生成器
    /// </summary>
    public class BPAUniqueIdBulder
    {
        class Generator
        {
            // should be between 40 (34 years) and 42 (139 years)
            internal const int NumberOfTimeBits = 42;

            // should be between 0 (single generator) and 10 (1024 generators)
            internal const int NumberOfGeneratorIdBits = 9;

            // should be 10 at least (4096 unique ids per millisecond per generator)
            internal const int NumberOfSequenceBits = 64 - NumberOfTimeBits - NumberOfGeneratorIdBits;

            private readonly byte[] _buffer = new byte[8];
            private readonly byte[] _timeBytes = new byte[8];
            private readonly byte[] _idBytes = new byte[2];
            private readonly byte[] _sequenceBytes = new byte[2];
            private readonly int _maxSequence = (int) Math.Pow(2, NumberOfSequenceBits) - 1;
            private readonly DateTime _start;

            private short _sequence;
            private long _previousTime;

            /// <summary>
            /// Instantiate the generator. Each Generator should have its own ID, so you can
            /// use multiple Generator instances in a cluster. All generated IDs are unique
            /// provided the start date newer changes. I recommend to choose January 1, 2013.
            /// </summary>
            public Generator(short generatorId, DateTime start)
            {
                if (generatorId < 0 || generatorId >= Math.Pow(2, NumberOfGeneratorIdBits))
                {
                    var msg = string.Format(CultureInfo.InvariantCulture,
                        "generator id must be between 0 (inclusive) and {0} (exclusive).",
                        Math.Pow(2, NumberOfGeneratorIdBits));
                    throw new ArgumentException(msg, "generatorId");
                }

                if (start > DateTime.Today)
                {
                    throw new ArgumentException("start date must not be in the future.", "start");
                }

                CalculateIdBytes(generatorId);
                _start = start;
            }

            /// <summary>
            /// Can generate up to 4096 different IDs per millisecond.
            /// </summary>
            public string Next()
            {
                SpinToNextSequence();
                WriteValuesToByteArray(_buffer, _previousTime, _sequence);

                return Convert.ToBase64String(_buffer);
            }

            public long NextLong()
            {
                SpinToNextSequence();
                WriteValuesToByteArray(_buffer, _previousTime, _sequence);

                Array.Reverse(_buffer);
                return BitConverter.ToInt64(_buffer, 0);
            }

            /// <summary>
            /// WriteValuesToByteArray
            /// </summary>
            /// <param name="target"></param>
            /// <param name="time"></param>
            /// <param name="sequence"></param>
            internal unsafe void WriteValuesToByteArray(byte[] target, long time, short sequence)
            {
                fixed (byte* arrayPointer = target)
                {
                    *(long*) arrayPointer = 0;
                }

                fixed (byte* arrayPointer = _timeBytes)
                {
                    *(long*) arrayPointer = time << (64 - NumberOfTimeBits);
                }

                fixed (byte* arrayPointer = _sequenceBytes)
                {
                    *(short*) arrayPointer = sequence;
                }

                WriteValuesToByteArray(target, _timeBytes, _idBytes, _sequenceBytes);
            }

            private unsafe void CalculateIdBytes(short id)
            {
                fixed (byte* arrayPointer = _idBytes)
                {
                    *(short*) arrayPointer = (short) (id << (8 - ((64 - NumberOfSequenceBits) % 8)));
                }
            }

            [MethodImpl(MethodImplOptions.AggressiveInlining)]
            private void WriteValuesToByteArray(byte[] target, byte[] time, byte[] id, byte[] sequence)
            {
                target[0] = (byte) (target[0] | time[7]);
                target[1] = (byte) (target[1] | time[6]);
                target[2] = (byte) (target[2] | time[5]);
                target[3] = (byte) (target[3] | time[4]);
                target[4] = (byte) (target[4] | time[3]);
                target[5] = (byte) (target[5] | time[2]);
                target[6] = (byte) (target[6] | time[1]);
                target[7] = (byte) (target[7] | time[0]);
                target[5] = (byte) (target[5] | id[1]);
                target[6] = (byte) (target[6] | id[0]);
                target[6] = (byte) (target[6] | sequence[1]);
                target[7] = (byte) (target[7] | sequence[0]);
            }

            private void SpinToNextSequence()
            {
                var time = GetTime();

                while (time == _previousTime && _sequence >= _maxSequence)
                {
                    time = GetTime();
                }

                _sequence = time == _previousTime ? (short) (_sequence + 1) : (short) 0;
                _previousTime = time;
            }

            [MethodImpl(MethodImplOptions.AggressiveInlining)]
            private long GetTime()
            {
                return (long) (DateTime.UtcNow - _start).TotalMilliseconds;
            }
        }

        private readonly static DateTime _startTime = new DateTime(2019, 1, 1, 0, 0, 0);
        private readonly static short _generatorIdMax = 512;
        private static HashSet<long> _idCache = null;
        private static DateTime? _idCacheCreateTime = null;
        private static object _getIdLock = new object();

        /// <summary>
        /// 默认的实例
        /// </summary>
        private readonly static BPAUniqueIdBulder Default = new BPAUniqueIdBulder();

        /// <summary>
        /// generatorId
        /// </summary>
        private short _generatorId = 0;

        /// <summary>
        /// 生成器
        /// </summary>
        private Generator _generator = null;

        /// <summary>
        /// 构造方法
        /// </summary>
        public BPAUniqueIdBulder() : this(System.Net.Dns.GetHostName())
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="hostflag"></param>
        public BPAUniqueIdBulder(string hostflag)
        {
            if (!string.IsNullOrEmpty(hostflag))
            {
                _generatorId = (short) (Math.Abs(hostflag.GetHashCode()) % _generatorIdMax);
            }

            this._generator = new Generator(_generatorId, _startTime);
        }

        /// <summary>
        /// GetId
        /// </summary>
        /// <returns></returns>
        private long GetId()
        {
            lock (_getIdLock)
            {
                var id = this._generator.NextLong();
                if (_idCacheCreateTime == null
                    || _idCache == null
                    || (DateTime.Now - _idCacheCreateTime.Value).TotalSeconds > 5)
                {
                    _idCache = new HashSet<long>();
                    _idCacheCreateTime = DateTime.Now;
                    _idCache.Add(id);
                    return id;
                }
                else
                {
                    if (_idCache.Contains(id))
                    {
                        return GetId();
                    }
                    else
                    {
                        _idCache.Add(id);
                        return id;
                    }
                }
            }
        }

        /// <summary>
        /// 获取下一个ID
        /// </summary>
        /// <returns></returns>
        public static long NextLong()
        {
            return Default.GetId();
        }
    }

    /// <summary>
    /// ID转换器
    /// </summary>
    public static class BPAUniqueIdConverter
    {
        /// <summary>
        /// IdConverter
        /// </summary>
        static class IdConverter
        {
            /// <summary>
            /// ie AAC3UcJAMAA -> 201561779220480
            /// </summary>
            public static long ToLong(string id)
            {
                var bytes = Convert.FromBase64String(id);
                Array.Reverse(bytes);
                return BitConverter.ToInt64(bytes, 0);
            }

            /// <summary>
            /// ie 201561779220480 -> AAC3UcJAMAA
            /// </summary>
            public static string ToString(long id)
            {
                var bytes = BitConverter.GetBytes(id);
                Array.Reverse(bytes);
                return Convert.ToBase64String(bytes);
            }
        }

        /// <summary>
        /// ToLong
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public static long ToLong(string id)
        {
            if (string.IsNullOrEmpty(id))
                throw new ArgumentNullException("id");
            //=，+在URL中传输时可能被误解成其它含义，所以必须转义
            id = id.Replace("~", "=").Replace("-", "+");
            return IdConverter.ToLong(id);
        }

        /// <summary>
        /// ToString
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public static string ToString(long id)
        {
            var re = IdConverter.ToString(id);
            //=，+在URL中传输时可能被误解成其它含义，所以必须转义
            return re.Replace("=", "~").Replace("+", "-");
        }
    }
}