﻿namespace BPA.Component.Extensions
{
    /// <summary>
    /// 日期扩展
    /// </summary>
    public static class DateTimeExtension
    {
        /// <summary>
        /// 1970-01-01 本地时区
        /// </summary>
        public static readonly DateTime DateTime1970 =
            TimeZoneInfo.ConvertTime(new System.DateTime(1970, 1, 1), TimeZoneInfo.Utc, TimeZoneInfo.Local);

        /// <summary>
        /// 转成json时间,返回的时从1970-01-01 到 dateTime 相差的毫秒
        /// </summary>
        /// <param name="dateTime">日期</param>
        /// <returns></returns>
        public static long ToJsTime(this DateTime dateTime)
        {
            long timeStamp = (long) (dateTime - DateTime1970).TotalMilliseconds; // 相差毫秒数
            return timeStamp;
        }

        /// <summary>
        /// 转成json时间,返回的时从1970-01-01 到 dateTime 相差的秒,本地时区
        /// </summary>
        /// <param name="dateTime">日期</param>
        /// <returns></returns>
        public static long ToJsTimeTotalSeconds(this DateTime dateTime)
        {
            long timeStamp = (long) (dateTime - DateTime1970).TotalSeconds; // 相差秒数
            return timeStamp;
        }

        /// <summary>
        /// JsTime 转成 DateTime ,本地时区
        /// </summary>
        /// <param name="jsTime"></param>
        /// <returns></returns>
        public static DateTime JsTimeToDateTime(this long jsTime)
        {
            DateTime dt = DateTime1970.AddMilliseconds(jsTime);
            return dt;
        }

        /// <summary>
        /// 计算日期是当前的第几周
        /// </summary>
        /// <param name="dateTime"></param>
        /// <param name="finalDateTime"></param>
        /// <returns></returns>
        public static int WeekOfMonth(this DateTime dateTime, out DateTime finalDateTime)
        {
            finalDateTime = dateTime;
            int dayInMonth = dateTime.Day;
            //本周第一天
            var firtDayWeek = FirstWeekDay(dateTime);
            //本月第一天  
            DateTime firstDayMonth = dateTime.AddDays(1 - dateTime.Day);
            //本月第一天周几
            int weekFirstDayMonth = (int) firstDayMonth.DayOfWeek == 0 ? 7 : (int) firstDayMonth.DayOfWeek;

            if (dayInMonth < 7 && firstDayMonth > firtDayWeek)
            {
                return WeekOfMonth(firtDayWeek, out finalDateTime);
            }

            //离周末还差几天
            int firstWeekEndDay = 7 - (weekFirstDayMonth);

            if (firstWeekEndDay < 7)
            {
                dayInMonth = dayInMonth - firstWeekEndDay;
            }

            int WeekNumInMonth = ((dayInMonth % 7) == 0
                ? (dayInMonth / 7 - 1)
                : (dayInMonth / 7)) + 1;
            return WeekNumInMonth;
        }

        /// <summary>
        /// 计算日期是当前的第几周（限定当前月）
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static int WeekOfLimitMonth(this DateTime dateTime)
        {
            int dayInMonth = dateTime.Day;
            //本月第一天  
            DateTime firstDay = dateTime.AddDays(1 - dateTime.Day);
            //本月第一天是周几  
            int weekday = (int) firstDay.DayOfWeek == 0 ? 7 : (int) firstDay.DayOfWeek;
            //本月第一周有几天  
            int firstWeekEndDay = 7 - (weekday - 1);
            //当前日期和第一周之差  
            int diffday = dayInMonth - firstWeekEndDay;
            diffday = diffday > 0 ? diffday : 1;
            //当前是第几周,如果整除7就减一天  
            int WeekNumInMonth = ((diffday % 7) == 0
                ? (diffday / 7 - 1)
                : (diffday / 7)) + 1 + (dayInMonth > firstWeekEndDay ? 1 : 0);
            return WeekNumInMonth;
        }

        /// <summary>
        /// 计算日期的当前周天数（限定当前月）
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static int WeekDaysLimitMonth(this DateTime dateTime)
        {
            var day = (int) dateTime.DayOfWeek;

            var count = 1;
            for (var i = day - 1; i >= 0; i--)
            {
                if (dateTime.AddDays(i - day).Month == dateTime.Month)
                {
                    count++;
                }
            }

            for (var i = day + 1; i <= 6; i++)
            {
                if (dateTime.AddDays(i - day).Month == dateTime.Month)
                {
                    count++;
                }
            }

            return count;
        }

        /// <summary>
        /// 计算日期的当前周的第一天
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static DateTime FirstWeekDay(this DateTime dateTime)
        {
            var day = (int) dateTime.DayOfWeek;
            return day > 0 ? dateTime.AddDays(-day) : dateTime;
        }

        /// <summary>
        /// 计算日期的当前周的第一天 （限定当前月）
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static DateTime FirstWeekDayLimitMonth(this DateTime dateTime)
        {
            var day = (int) dateTime.DayOfWeek;

            var count = 0;
            for (var i = day - 1; i >= 0; i--)
            {
                if (dateTime.AddDays(i - day).Month == dateTime.Month)
                {
                    count++;
                }
            }

            return count > 0 ? dateTime.AddDays(-count) : dateTime;
        }

        /// <summary>
        /// 计算日期的当前周的第一天
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static DateTime LastWeekDay(this DateTime dateTime)
        {
            var day = (int) dateTime.DayOfWeek;
            return day < 6 ? dateTime.AddDays(6 - day) : dateTime;
        }

        /// <summary>
        /// 计算日期的当前周的最后一天 （限定当前月）
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static DateTime LastWeekDayLimitMonth(this DateTime dateTime)
        {
            var day = (int) dateTime.DayOfWeek;

            var count = 0;
            for (var i = day + 1; i <= 6; i++)
            {
                if (dateTime.AddDays(i - day).Month == dateTime.Month)
                {
                    count++;
                }
            }

            return count > 0 ? dateTime.AddDays(count) : dateTime;
        }

        /// <summary>
        /// 获取日期所在季度
        /// </summary>
        /// <param name="dateTime"></param>
        /// <returns></returns>
        public static int ToQuarter(this DateTime dateTime)
        {
            return (dateTime.Month - 1) / 3 + 1;
        }
    }
}