﻿using System.ComponentModel;
using System.Diagnostics.CodeAnalysis;
using System.Web;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// 字典扩展
    /// </summary>
    public static class DictionaryExtension
    {
        /// <summary>
        /// 将字典转成URL参数格式,key=value%26key1=value1
        /// </summary>
        /// <param name="source"></param>
        /// <param name="isEncode">是否编码,默认false</param>
        /// <returns></returns>
        public static string ToUrlParams<TValue>(this Dictionary<string, TValue> source, bool isEncode = false)
        {
            if (isEncode)
                return string.Join("&",
                    source.Select(it =>
                        $"{it.Key}={HttpUtility.UrlEncode(it.Value.IsNull() ? string.Empty : it.Value.ToString())}"));
            return string.Join("&",
                source.Select(it => $"{it.Key}={(it.Value.IsNull() ? string.Empty : it.Value.ToString())}"));
        }

        /// <summary>
        /// 将字典转成URL参数格式,key=value%26key1=value1,Key 按照 ASCII码从 排序
        /// </summary>
        /// <param name="source"></param>
        /// <param name="sortType">排序,默认Asc</param>
        /// <param name="isEncode">是否编码,默认false</param>
        /// <returns></returns>
        public static string ToUrlParamsByASCIISort<TValue>(this Dictionary<string, TValue> source,
            ASCIISortType sortType = ASCIISortType.Asc, bool isEncode = false)
        {
            Dictionary<string, TValue> pairs = sortType == ASCIISortType.Asc
                ? source.OrderBy(x => x.Key, new ComparerString()).ToDictionary(x => x.Key, y => y.Value)
                : source.OrderByDescending(x => x.Key, new ComparerString()).ToDictionary(x => x.Key, y => y.Value);
            return pairs.ToUrlParams(isEncode);
        }

        /// <summary>
        ///  获取所有的key
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="source"></param>
        /// <returns></returns>
        public static List<TKey> AllKeys<TKey, TValue>(this Dictionary<TKey, TValue> source)
        {
            var keys = new List<TKey>();
            keys.AddRange(source.Keys);
            return keys;
        }

        /// <summary>
        /// 安全的each  key
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="source"></param>
        /// <param name="each"></param>
        public static void EachKeySafety<TKey, TValue>(this Dictionary<TKey, TValue> source, Action<TKey> each)
        {
            var keys = source.AllKeys();
            keys.Each(key => each(key));
        }
    }

    /// <summary>
    /// 排序方式
    /// </summary>
    public enum ASCIISortType
    {
        /// <summary>
        /// 升序,从小到大
        /// </summary>
        [Description("升序,从小到大")] Asc = 0,

        /// <summary>
        /// 降序,从大到小
        /// </summary>
        [Description("降序,从大到小")] Desc = 1,
    }

    /// <summary>
    /// ComparerString
    /// </summary>
    internal class ComparerString : IComparer<string>
    {
        /// <summary>
        /// Compare
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <returns></returns>
        public int Compare([NotNull]string x, [NotNull]string y)
        {
            return string.CompareOrdinal(x, y);
        }
    }
}