﻿using System.ComponentModel;
using System.Reflection;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// 枚举扩展类
    /// </summary>
    public static class EnumExtension
    {
        /// <summary>
        /// 获取枚举第一个描述信息,获取不到时返回空字符串string.Empty, 枚举字段须使用<see cref="DescriptionAttribute"/> 
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string GetDescribe(this Enum source)
        {
            var attribute = source.GetFristAttribute<DescriptionAttribute>();
            if (attribute == null)
                return string.Empty;
            return attribute.Description ?? string.Empty;
        }

        /// <summary>
        /// 获取枚举特定的Attribute，返回第一个
        /// </summary>
        /// <typeparam name="TAttribute">标记类型</typeparam>
        /// <param name="source"></param>
        /// <returns></returns>
        public static TAttribute GetFristAttribute<TAttribute>(this Enum source) where TAttribute : Attribute
        {
            var attribute = source.GetType().GetField(source.ToString()).GetCustomAttribute(typeof(TAttribute), true);
            if (attribute == null)
                return default;
            return (TAttribute)attribute;
        }

        /// <summary>
        /// 获取枚举特定的Attribute,返回集合
        /// </summary>
        public static List<TAttribute> GetAttributes<TAttribute>(this Enum source) where TAttribute : Attribute
        {
            var result = new List<TAttribute>();
            var attributes = source.GetType().GetField(source.ToString()).GetCustomAttributes(typeof(TAttribute), true);
            if (attributes == null || attributes.Length == 0)
                return new List<TAttribute>();
            return attributes.Select(it => (TAttribute)it).ToList();
        }

        /// <summary>
        /// int 转 枚举
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="enumIntValue">枚举int值</param>
        /// <returns></returns>
        public static TEnum ToParse<TEnum>(this int enumIntValue) where TEnum : Enum
        {
            return (TEnum)Enum.ToObject(typeof(TEnum), enumIntValue);
        }

        /// <summary>
        /// short 转 枚举
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="enumShortValue">枚举int值</param>
        /// <returns></returns>
        public static TEnum ToParse<TEnum>(this short enumShortValue) where TEnum : Enum
        {
            return (TEnum)Enum.ToObject(typeof(TEnum), enumShortValue);
        }

        /// <summary>
        /// string 转枚举
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="enumStringValue">枚举string字段</param>
        /// <returns></returns>
        public static TEnum ToParse<TEnum>(this string enumStringValue) where TEnum : Enum
        {
            return (TEnum)Enum.Parse(typeof(TEnum), enumStringValue,true);
        }

        /// <summary>
        /// 获取所有的枚举字段
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <returns></returns>
        public static List<TEnum> GetFields<TEnum>(this TEnum source) where TEnum : Enum
        {
            List<TEnum> list = new List<TEnum>();
            foreach (object obj in Enum.GetValues(typeof(TEnum)))
            {
                list.Add((TEnum)obj);
            }
            return list;
        }

        /// <summary>
        /// 列举枚举所有字段和对应描述 转化成字典
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="source"></param>
        /// <returns></returns>
        public static Dictionary<TEnum, string> GetFieldsToDic<TEnum>(this TEnum source) where TEnum : Enum
        {
            var keyValuePairs = new Dictionary<TEnum, string>();
            foreach (TEnum item in source.GetFields())
            {
                keyValuePairs.Add(item, item.GetDescribe());
            }
            return keyValuePairs;
        }

        /// <summary>
        /// 位域运算，拆解枚举值
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="source"></param>
        /// <returns></returns>
        public static List<TEnum> GetFlags<TEnum>(this TEnum source) where TEnum : Enum
        {
            return source.GetFields().Where(a => source.HasFlag(a)).ToList();

        }

        /// <summary>
        /// 将多个枚举 合并为一个枚举，必须有Flags标记的枚举类
        /// </summary>
        /// <typeparam name="TEnum"></typeparam>
        /// <param name="source"></param>
        /// <returns></returns>
        public static TEnum CombineFlag<TEnum>(this IEnumerable<TEnum> source) where TEnum : Enum
        {
            ulong en = 0;
            foreach (TEnum s in source)
            {
                en |= Convert.ToUInt64(s);
            }
            return (TEnum)Enum.ToObject(typeof(TEnum), Convert.ToUInt64(en));
        }

        /// <summary>
        /// 合并为一个枚举 ,必须有Flags标记的枚举类, 追加值
        /// </summary>
        public static TEnum CombineSet<TEnum>(this TEnum source, TEnum value) where TEnum : Enum
        {
            return (TEnum)Enum.ToObject(source.GetType(), Convert.ToUInt64(source) | Convert.ToUInt64(value));
        }

        /// <summary>
        /// 合并为一个枚举,必须有Flags标记的枚举类, 追加值
        /// </summary>
        public static TEnum CombineSet<TEnum>(this TEnum source, params TEnum[] values) where TEnum : Enum
        {
            values?.Each(it => source = source.CombineSet(it));
            return source;
        }

        /// <summary>
        /// 合并为一个枚举,必须有Flags标记的枚举类, 追加值
        /// </summary>
        public static TEnum CombineSet<TEnum>(this TEnum source, IEnumerable<TEnum> values) where TEnum : Enum
        {
            values?.Each(it => source = source.CombineSet(it));
            return source;
        }
    }
}
