﻿using System.Globalization;
using System.Security.Cryptography;
using System.Text;
using System.Web;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// string extension
    /// </summary>
    public static class StringExtension
    {
        /// <summary>
        /// 判断字符串 不为null或空字符串
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static bool IsNotEmpty(this string source)
        {
            return !string.IsNullOrEmpty(source);
        }

        /// <summary>
        /// 判断字符串 为null或空字符串
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static bool IsEmpty(this string source)
        {
            if (source == null)
                return true;
            return string.IsNullOrEmpty(source.Trim());
        }

        /// <summary>
        /// Trim,如果为空返回string.Empty
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToTrim(this string source)
        {
            if (source == null)
                return string.Empty;
            return source.Trim();
        }

        /// <summary>
        /// tolong
        /// </summary>
        /// <param name="source"></param>
        /// <param name="defaultVal"></param>
        /// <returns></returns>
        public static long ToLong(this string source, long defaultVal = 0)
        {
            if (long.TryParse(source, out long result))
                return result;
            return defaultVal;
        }

        /// <summary>
        /// to long or null
        /// </summary>
        /// <param name="source"></param>
        /// <param name="defaultVal"></param>
        /// <returns></returns>
        public static long? ToLongNull(this string source, long? defaultVal = null)
        {
            if (long.TryParse(source, out long result))
                return result;
            return defaultVal;
        }


        /// <summary>
        /// toUlong
        /// </summary>
        /// <param name="source"></param>
        /// <param name="defaultVal"></param>
        /// <returns></returns>
        public static ulong ToUlong(this string source, ulong defaultVal = 0)
        {
            if (ulong.TryParse(source, out ulong result))
                return result;
            return defaultVal;
        }

        /// <summary>
        /// to toUlong or null
        /// </summary>
        /// <param name="source"></param>
        /// <param name="defaultVal"></param>
        /// <returns></returns>
        public static ulong? ToUlongNull(this string source, ulong? defaultVal = null)
        {
            if (ulong.TryParse(source, out ulong result))
                return result;
            return defaultVal;
        }

        /// <summary>
        /// 字符串转换成字典
        /// </summary>
        /// <param name="param">url param 格式字符串</param>
        /// <param name="isDecode">是否解码,默认不解码</param>
        /// <returns></returns>
        public static Dictionary<string, string> UrlParamsToDic(this string param, bool isDecode = false)
        {
            var dic = new Dictionary<string, string>();
            var arr = param.Split('&');
            foreach (var item in arr)
            {
                var itemArr = item.Split('=');
                if (itemArr.Length <= 1)
                {
                    continue;
                }

                var vuale = itemArr[1];
                if (vuale != null && isDecode)
                    vuale = HttpUtility.UrlDecode(vuale);
                dic.Add(itemArr[0], vuale);
            }

            return dic;
        }

        /// <summary>
        /// bytes 转 16进制 字符串转
        /// </summary>
        public static string ToHexString(this byte[] bytes)
        {
            string result = string.Empty;
            for (int i = 0; i < bytes.Length; i++) //逐字节变为16进制字符
            {
                result += Convert.ToString(bytes[i], 16).PadLeft(2, '0');
            }

            return result;
        }

        /// <summary>
        /// 字符串转Base64
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="encode">编码格式，等于null 代表UTF8</param>
        /// <returns></returns>
        public static string ToBase64String(this string str, Encoding encode = null)
        {
            encode = encode ?? Encoding.UTF8;
            byte[] byteArr = encode.GetBytes(str);
            return Convert.ToBase64String(byteArr);
        }


        /// <summary>
        /// ulong 转16进制
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToH16(this ulong source)
        {
            return source.ToString("x");
        }

        /// <summary>
        /// ulong 转16进制
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToH16(this ulong? source)
        {
            if (source == null)
                return null;
            return source.Value.ToString("x");
        }

        /// <summary>
        /// 16进制字符串转ulong
        /// </summary>
        /// <param name="h16"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public static long H16Tolong(this string h16, long defaultValue = 0)
        {
            try
            {
                if (h16.IsEmpty())
                    return defaultValue;
                return Convert.ToInt64(h16, 16);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return defaultValue;
            }
        }

        /// <summary>
        /// 16进制字符串转ulong
        /// </summary>
        /// <param name="h16"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public static long? H16Tolong(this string h16, long? defaultValue = null)
        {
            try
            {
                if (h16.IsEmpty())
                    return defaultValue;
                return Convert.ToInt64(h16, 16);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return defaultValue;
            }
        }

        /// <summary>
        /// 转成骆峰命名
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToCamelCase(this string source)
        {
            try
            {
                if (source.IsEmpty())
                {
                    return source;
                }

                if (string.IsNullOrEmpty(source) || !char.IsUpper(source[0]))
                {
                    return source;
                }

                char[] array = source.ToCharArray();
                for (int i = 0; i < array.Length && (i != 1 || char.IsUpper(array[i])); i++)
                {
                    bool flag = i + 1 < array.Length;
                    if (i > 0 && flag && !char.IsUpper(array[i + 1]))
                    {
                        if (char.IsSeparator(array[i + 1]))
                        {
                            array[i] = ToLower(array[i]);
                        }

                        break;
                    }

                    array[i] = ToLower(array[i]);
                }

                return new string(array);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return source;
            }
        }

        /// <summary>
        /// 转成骆峰命名
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToCamelCase(this Enum source)
        {
            try
            {
                var key = source.ToString();
                return key.ToCamelCase();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return source.ToString();
            }
        }

        /// <summary>
        /// 转成帕斯卡命名
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ToPasscal(this string source)
        {
            try
            {
                if (source.IsEmpty())
                {
                    return source;
                }

                return source.Substring(0, 1).ToUpper() + source.Substring(1);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return source;
            }
        }

        /// <summary>
        /// 转时间
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static DateTime ToParseDateTime(this string str)
        {
            DateTime.TryParse(str, out DateTime test);
            return test;
        }

        /// <summary>
        /// 转时间
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static DateTime? ToParseNullableDateTime(this string str)
        {
            if (DateTime.TryParse(str, out DateTime test))
            {
                return test;
            }

            return null;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        private static char ToLower(char c)
        {
            c = char.ToLower(c, CultureInfo.InvariantCulture);
            return c;
        }

        /// <summary>
        ///  字符串SHA1加密
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static string ToSHA1(this string str)
        {
            try
            {
#pragma warning disable 618
                SHA1 sha1 = new SHA1CryptoServiceProvider(); //创建SHA1对象
#pragma warning restore 618
                byte[] bytes_in = Encoding.UTF8.GetBytes(str); //将待加密字符串转为byte类型
                byte[] bytes_out = sha1.ComputeHash(bytes_in); //Hash运算
                sha1.Dispose(); //释放当前实例使用的所有资源
                string result = BitConverter.ToString(bytes_out); //将运算结果转为string类型
                result = result.Replace("-", "").ToLower();
                return result;
            }
            catch (Exception)
            {
            }

            return "";
        }

        /// <summary>
        /// HTML 编码
        /// <![CDATA[
        ///  html.Replace("<", "&lt;").Replace(">", "&gt;");
        /// ]]>
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public static string HtmlEncode(this string html)
        {
            if (html == null) return html;
            return html.Replace("<", "&lt;").Replace(">", "&gt;");
        }

        /// <summary>
        /// sql注入参数  过滤
        /// </summary>
        /// <param name="sqlParameterValue"></param>
        /// <returns></returns>
        public static string SqlParameterEscape(this string sqlParameterValue)
        {
            if (sqlParameterValue == null) return sqlParameterValue;
            return sqlParameterValue.Replace(",", string.Empty).Replace("or", string.Empty, true, null)
                .Replace("--", string.Empty);
        }
    }
}