﻿using System.ComponentModel;
using System.Reflection;

namespace BPA.Component.Extensions
{
    /// <summary>
    /// 类型扩展
    /// </summary>
    public static class TypeExtension
    {
        /// <summary>
        /// 类型转字段
        /// </summary>
        /// <param name="type">类型</param>
        /// <remarks>
        /// Key:属性名;Value:属性描述
        /// </remarks>
        /// <returns></returns>
        public static Dictionary<string, string> ToDictionary(this Type type)
        {
            Dictionary<string, string> dictionary = new Dictionary<string, string>();
            foreach (var item in type.GetProperties())
            {
                DescriptionAttribute[] descAttr = (DescriptionAttribute[])item.GetCustomAttributes(typeof(DescriptionAttribute), false);
                string description = descAttr.FirstOrDefault()?.Description ?? string.Empty;
                dictionary.Add(item.Name, description);
            }
            return dictionary;
        }

        /// <summary>
        /// 获取属性的特定标记
        /// </summary>
        /// <typeparam name="TAttribute"></typeparam>
        /// <param name="propertyInfo"></param>
        /// <returns></returns>
        public static List<TAttribute> GetAttributes<TAttribute>(this PropertyInfo propertyInfo) where TAttribute : Attribute
        {
            return propertyInfo.GetCustomAttributes(typeof(TAttribute), false).Select(it => (TAttribute)it).ToList();
        }

        /// <summary>
        /// 获取属性的特定标记,获取不到时返回null
        /// </summary>
        /// <typeparam name="TAttribute"></typeparam>
        /// <param name="propertyInfo"></param>
        /// <returns></returns>
        public static TAttribute GetAttribute<TAttribute>(this PropertyInfo propertyInfo) where TAttribute : Attribute
        {
            var attribute = propertyInfo.GetCustomAttribute(typeof(TAttribute), false);
            if (attribute == null)
                return default;
            return (TAttribute)attribute;
        }

        /// <summary>
        /// 获取type完整的描述名称
        /// </summary>
        /// <param name="type"></param>
        /// <param name="isReplacSystemeNamespace">默认替换 system命令空间</param>
        /// <param name="replacSystemeNamespaces">其他需要替换的命名空间</param>
        /// <returns></returns>
        public static string GetDescribe(this Type type, bool isReplacSystemeNamespace = true, params string[] replacSystemeNamespaces)
        {
            string ret = string.Empty;
            if (type.IsGenericType)//是否是泛型
            {
                string name = type.FullName.Split('`')[0]; 
                ret += ReplacSystemeNamespace(name, isReplacSystemeNamespace, replacSystemeNamespaces);
                ret += "<";
                var tps = type.GetGenericArguments();//返回泛型类型数组
                for (int i = 0; i < tps.Length; i++)
                {
                    ret += GetDescribe(tps[i], isReplacSystemeNamespace, replacSystemeNamespaces);
                    if (i != tps.Length - 1)
                        ret += ",";
                }
                ret += ">";
            }
            else
                ret = ReplacSystemeNamespace(type.FullName, isReplacSystemeNamespace, replacSystemeNamespaces);
            if (type.FullName.IndexOf("+", StringComparison.Ordinal) > 0 && type.FullName.IndexOf("[", StringComparison.Ordinal) > 0)
            {
                var arr = type.FullName.Split('`');
                ret += "." + arr[1].Split('[')[0].Replace("1", string.Empty).Replace("+", string.Empty);
            }
            return ret;
        }

        /// <summary>
        /// 处理需要替换的命令空间
        /// </summary>
        /// <param name="typeName"></param>
        /// <param name="isReplacSystemeNamespace"></param>
        /// <param name="replacSystemeNamespaces"></param>
        /// <returns></returns>
        private static string ReplacSystemeNamespace(string typeName, bool isReplacSystemeNamespace = true, params string[] replacSystemeNamespaces)
        {
            if (isReplacSystemeNamespace && typeName.StartsWith("System"))
            {
                typeName = typeName.Substring(typeName.LastIndexOf(".") + 1);
                typeName = typeName.Replace("UInt64", "ulong");
                typeName = typeName.Replace("Int64", "long");
                typeName = typeName.Replace("Int32", "int");
                typeName = typeName.Replace("String", "string");
                typeName = typeName.Replace("Boolean", "bool");
                typeName = typeName.Replace("Decimal", "decimal");
                typeName = typeName.Replace("Double", "double");
                typeName = typeName.Replace("Single", "float");
            }
            foreach (var ns in replacSystemeNamespaces)
                typeName = typeName.Replace(ns, string.Empty);
            typeName = typeName.Replace("+", ".");
            return typeName;
        }

    }
}

