﻿using System.Net;
using System.Text;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog;
using NLog.Config;
using NLog.Layouts;
using NLog.Targets;
using NLog.Targets.Wrappers;
using NLog.Targets.ElasticSearch;
using NLog.Web;
using LogLevel = Microsoft.Extensions.Logging.LogLevel;

namespace BPA.Component.LogClient.Extensions
{
    /// <summary>
    /// BPALoggerConfigurationExtensions
    /// </summary>
    public static class BPALoggerConfigurationExtensions
    {
        /// <summary>
        /// UseBPALog,
        /// </summary>
        /// <param name="services"></param>
        /// <param name="isTargetFile"></param>
        /// <param name="elasticSearchConfig"></param>
        /// <returns></returns>
        public static IServiceCollection AddBPALog(this IServiceCollection services, bool isTargetFile = false)
        {
            services.AddHttpContextAccessor();
            services.AddLogging(loggingBuilder =>
            {
                loggingBuilder.AddNLog(_ => LogManager.LogFactory);

                var isDev = Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT") == "Development";
                if (!isDev)
                {
                    loggingBuilder.AddFilter("System", LogLevel.Error);
                    loggingBuilder.AddFilter("Microsoft", LogLevel.Error);
                }
                else
                {
                    loggingBuilder.AddFilter("System", LogLevel.Warning);
                    loggingBuilder.AddFilter("Microsoft", LogLevel.Warning);
                }

                loggingBuilder.SetMinimumLevel(LogLevel.Debug);
            });

            ConfigureLog(isTargetFile);

            return services;
        }

        /// <summary>
        /// Configure Log
        /// </summary>
        /// <param name="isTargetFile"></param>
        private static void ConfigureLog(bool isTargetFile)
        {
            var appName = AppDomain.CurrentDomain.FriendlyName;
            var appServerIp = GetLocalIp();
            var environment = Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT");
            var appBaseLogDir = AppDomain.CurrentDomain.BaseDirectory;
            var isDev = environment is "Development" or "Testing";

            Console.WriteLine($"*BPA.logclient* configure log info: [APP_SERVER_IP]={appServerIp}");
            Console.WriteLine($"*BPA.logclient* configure log info: [APP_BASE_LOG_DIR]={appBaseLogDir}");
            Console.WriteLine($"*BPA.logclient* configure log info: [ASPNETCORE_ENVIRONMENT]={environment}");

            LogManager.ThrowConfigExceptions = isDev;
            LogManager.ThrowExceptions = isDev;

            var configuration = new LoggingConfiguration(LogManager.LogFactory);
            configuration.UseConsoleTarget();

            if (isTargetFile)
            {
                configuration.UseFileTarget(appBaseLogDir, appName);
            }

            LogManager.Configuration = configuration;
        }

        public static void UseConsoleTarget(this LoggingConfiguration loggingConfiguration)
        {
            var target = new ConsoleTarget
            {
                Name = "ConsoleTarget",
                AutoFlush = true,
                Layout = new SimpleLayout("${date} | ${level:uppercase=true} | ${logger}  " +
                                          "${newline}${message} " +
                                          "${newline}${exception:format=tostring}" +
                                          "${newline}${all-event-properties}   " +
                                          "${newline}${aspnet-request-url:IncludePort=true:IncludeQueryString=true:UseRawTarget=true} " +
                                          "${newline}")
            };

            loggingConfiguration.AddTarget(target);

#if DEBUG
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Trace,
                NLog.LogLevel.Error,
                target));
#else
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Error,
                NLog.LogLevel.Fatal,
                target));
#endif
        }

        public static void UseFileTarget(this LoggingConfiguration loggingConfiguration,
            string appBaseLogDir,
            string appName)
        {
            var layout = CreateJsonLayout();
            var appLogDir = Path.Join(appBaseLogDir, appName);
            if (!Directory.Exists(appLogDir))
            {
                Directory.CreateDirectory(appLogDir);
            }

            var fileName = "${date:format=yyyy-MM-dd-HH}-" + appName.GetHashCode() + "-${level}.log";
            fileName = Path.Join(appLogDir, fileName);

            Console.WriteLine($"*BPA.logclient* use file target log info: [FileName]={fileName}");

            var target = new FileTarget
            {
                Encoding = Encoding.UTF8,
                Name = "FileTarget",
                AutoFlush = true,
                FileName = fileName,
                Layout = layout,
                BufferSize = 10240,
                KeepFileOpen = true,
                CreateDirs = true,
                ConcurrentWrites = true,
                // DeleteOldFileOnStartup = true,
                EnableFileDelete = true,
            };

            var wrapper = new AsyncTargetWrapper
            {
                Name = "AsyncTargetWrapper",
                WrappedTarget = target,
                QueueLimit = 5000,
                OverflowAction = AsyncTargetWrapperOverflowAction.Discard
            };

            loggingConfiguration.AddTarget(wrapper);
#if DEBUG
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Trace,
                NLog.LogLevel.Error,
                wrapper));
#else
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Error,
                NLog.LogLevel.Fatal,
                wrapper));
#endif
        }

        public static void UseElkTarget(this IApplicationBuilder app, ElasticSearchConfig elasticSearchConfig, LoggingConfiguration loggingConfiguration)
        {
            if (elasticSearchConfig == null) return;

            var target = new ElasticSearchTarget();
            target.Uri = elasticSearchConfig.Url;
            target.Layout = CreateJsonLayout();
            target.Username = elasticSearchConfig.Username;
            target.Password = elasticSearchConfig.Password;
            var wrapper = new AsyncTargetWrapper
            {
                Name = "AsyncTargetWrapper",
                WrappedTarget = target,
                QueueLimit = 5000,
                OverflowAction = AsyncTargetWrapperOverflowAction.Discard
            };

            loggingConfiguration.AddTarget(wrapper);
#if DEBUG
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Trace,
                NLog.LogLevel.Error,
                wrapper));
#else
            loggingConfiguration.LoggingRules.Add(new LoggingRule("*",
                NLog.LogLevel.Error,
                NLog.LogLevel.Fatal,
                wrapper));
#endif
        }

        private static JsonLayout CreateJsonLayout()
        {
            var appName = AppDomain.CurrentDomain.FriendlyName;
            var appServerIp = GetLocalIp();

            var layout = new JsonLayout();
            layout.Attributes.Add(new JsonAttribute("app", appName.ToLower()) {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("ip", appServerIp) {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("@timestamp", "${date:universalTime=true:format=yyyy-MM-ddTHH\\:mm\\:ss.fffZ}") {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("logLevel", "${level}") {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("file", "${callsite:filename=true}") {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("msg", "${message}") {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("exception", "${exception:format=tostring}") {EscapeUnicode = false});
            layout.Attributes.Add(new JsonAttribute("request.traceid", "${aspnet-traceidentifier:IgnoreActivityId=true}") {EscapeUnicode = false});
            return layout;
        }

        private static string GetLocalIp()
        {
            foreach (var ipAddress in Dns.GetHostEntry(Dns.GetHostName()).AddressList)
            {
                if (ipAddress.AddressFamily.ToString() == "InterNetwork")
                {
                    return ipAddress.ToString();
                }
            }

            return string.Empty;
        }
    }
}