﻿using System.Reflection;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Serializers;

namespace BPA.Component.MongoClient.Conventions
{

    /// <summary>
    /// EnumBsonSerializer
    /// </summary>
    /// <typeparam name="TEnum"></typeparam>
    public class EnumBsonSerializer<TEnum> : StructSerializerBase<TEnum>, IRepresentationConfigurable<EnumBsonSerializer<TEnum>> where TEnum : struct, Enum
    {
        /// <summary>
        /// Representation
        /// </summary>
        public BsonType Representation { set; get; }

        /// <summary>
        /// EnumBsonSerializer
        /// </summary>
        public EnumBsonSerializer() : this(BsonType.String)
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="EnumSerializer{TEnum}"/> class.
        /// </summary>
        /// <param name="representation">The representation.</param>
        public EnumBsonSerializer(BsonType representation)
        {
            switch (representation)
            {
                case 0:
                case BsonType.Int32:
                case BsonType.Int64:
                case BsonType.String:
                    break;

                default:
                    var message = string.Format("{0} is not a valid representation for an EnumSerializer.", representation);
                    throw new ArgumentException(message);
            }

            // don't know of a way to enforce this at compile time
            var enumTypeInfo = typeof(TEnum).GetTypeInfo();
            if (!enumTypeInfo.IsEnum)
            {
                var message = string.Format("{0} is not an enum type.", typeof(TEnum).FullName);
                throw new BsonSerializationException(message);
            }
            Representation = representation;
        }

        /// <summary>
        /// Deserialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <returns></returns>
        public override TEnum Deserialize(BsonDeserializationContext context, BsonDeserializationArgs args)
        {
            var bsonReader = context.Reader;
            var bsonType = bsonReader.GetCurrentBsonType();
            switch (bsonType)
            {
                case BsonType.Int32: return (TEnum)Enum.ToObject(typeof(TEnum), bsonReader.ReadInt32());
                case BsonType.Int64: return (TEnum)Enum.ToObject(typeof(TEnum), bsonReader.ReadInt64());
                case BsonType.Double: return (TEnum)Enum.ToObject(typeof(TEnum), (long)bsonReader.ReadDouble());
                case BsonType.String: return (TEnum)Enum.Parse(typeof(TEnum), bsonReader.ReadString(), true);
                default:
                    throw CreateCannotDeserializeFromBsonTypeException(bsonType);
            }
        }

        /// <summary>
        /// Serialize
        /// </summary>
        /// <param name="context"></param>
        /// <param name="args"></param>
        /// <param name="value"></param>
        public override void Serialize(BsonSerializationContext context, BsonSerializationArgs args, TEnum value)
        {
            var bsonWriter = context.Writer;

            switch (Representation)
            {
                case 0:
                    var underlyingTypeCode = Type.GetTypeCode(Enum.GetUnderlyingType(typeof(TEnum)));
                    if (underlyingTypeCode == TypeCode.Int64 || underlyingTypeCode == TypeCode.UInt64)
                    {
                        goto case BsonType.Int64;
                    }
                    else
                    {
                        goto case BsonType.Int32;
                    }
                case BsonType.Int32:
                    bsonWriter.WriteInt32(Convert.ToInt32(value));
                    break;
                case BsonType.Int64:
                    bsonWriter.WriteInt64(Convert.ToInt64(value));
                    break;
                case BsonType.String:
                    var val = value.ToString().ToCamelCase();
                    bsonWriter.WriteString(val);
                    break;
                default:
                    throw new BsonInternalException("Unexpected EnumRepresentation.");
            }
        }

        /// <summary>
        /// WithRepresentation
        /// </summary>
        /// <param name="representation"></param>
        /// <returns></returns>
        IBsonSerializer IRepresentationConfigurable.WithRepresentation(BsonType representation)
        {
            return WithRepresentation(representation);
        }

        /// <summary>
        /// WithRepresentation
        /// </summary>
        /// <param name="representation"></param>
        /// <returns></returns>
        public EnumBsonSerializer<TEnum> WithRepresentation(BsonType representation)
        {
            if (representation == Representation)
            {
                return this;
            }
            else
            {
                return new EnumBsonSerializer<TEnum>(representation);
            }
        }




    }
}
