﻿using System.Linq.Expressions;
using BPA.Component.MongoClient.Conventions;
using MongoDB.Bson;
using MongoDB.Bson.Serialization;
using MongoDB.Bson.Serialization.Conventions;
using MongoDB.Bson.Serialization.Serializers;
using MongoDB.Driver;
using MongoDB.Driver.Linq;
using Newtonsoft.Json.Linq;

namespace BPA.Component.MongoClient.Repository;

/// <summary>
/// MongoDb基础仓储
/// </summary>
public class BaseMongoDbRepository<TMongoDbClient, TDocument> : IBaseMongoDbRepository<TDocument> where TMongoDbClient : BaseMongoDbClient<TMongoDbClient>
    where TDocument : new()
{
    /// <summary>
    /// MongoClient
    /// </summary>
    public MongoDB.Driver.MongoClient _mongoDbClient { get; }

    /// <summary>
    /// _Database
    /// </summary>
    private readonly IMongoDatabase _database;

    /// <summary>
    /// _Collection
    /// </summary>
    private IMongoCollection<TDocument> _collection;

    /// <summary>
    /// 构造函数
    /// </summary> 
#pragma warning disable CS8618
    internal BaseMongoDbRepository()
#pragma warning restore CS8618
    {
    }

    /// <summary>
    /// 构造方法
    /// </summary>
    public BaseMongoDbRepository(BaseMongoDbClient<TMongoDbClient> mongoDbClient)
    {
        _mongoDbClient = new MongoDB.Driver.MongoClient(mongoDbClient.Config.ConnectionString);
        _database = _mongoDbClient.GetDatabase(mongoDbClient.CurrentDatabaseName);
        var serializer = new DateTimeSerializer(DateTimeKind.Local, BsonType.DateTime);
        BsonSerializer.RegisterSerializer(typeof(DateTime), serializer);
        BsonSerializer.RegisterSerializer(typeof(JObject), new JObjectSerializer());
        BsonSerializer.RegisterSerializer(typeof(JArray), new JArraySerializer());
        BsonSerializer.RegisterSerializer(typeof(Dictionary<string, object>), new DictionarySerializerSerializer());
        BsonSerializer.RegisterSerializer(typeof(decimal), new DecimalSerializer(BsonType.Decimal128));
        BsonSerializer.RegisterSerializer(typeof(decimal?), new NullableSerializer<decimal>(new DecimalSerializer(BsonType.Decimal128)));
        BsonSerializer.RegisterSerializationProvider(new BPASerializationProvider());

        ConventionRegistry.Register("IgnoreExtraElements",
            new ConventionPack
            {
                new IgnoreExtraElementsConvention(true), new BPACamelCaseElementNameConvention(),
                new EnumRepresentationConvention(BsonType.String)
            }, _ => true);
    }

    /// <summary>
    /// 插入一个
    /// </summary>
    /// <param name="document"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    public int Add<TDocument>(TDocument document)
    {
        if (document == null) return 0;
        var coll = _database.GetCollection<TDocument>(document.GetType().Name);
        coll.InsertOne(document);

        return 1;
    }


    /// <summary>
    /// 插入多个
    /// </summary>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    public int AddList<TDocument>(IEnumerable<TDocument> documents)
    {
        if (documents == null) return 0;
        var coll = _database.GetCollection<TDocument>(documents.First().GetType().Name);
        coll.InsertMany(documents);

        return 1;
    }

    /// <summary>
    /// 修改一个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <param name="options"></param>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    public UpdateResult UpdateOne<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression, UpdateOptions options = null)
    {
        if (document == null) throw new ArgumentNullException("null document");
        return _database.GetCollection<TDocument>(document.GetType().Name).UpdateOne(document, whereExpression, options);
    }

    /// <summary>
    /// 修改多个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <param name="options"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    public UpdateResult UpdateMany<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression, UpdateOptions options = null)
    {
        if (document == null) throw new ArgumentNullException("null document");
        return _database.GetCollection<TDocument>(document.GetType().Name).UpdateMany(document, whereExpression, options);
    }

    /// <summary>
    /// 删除一个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    public DeleteResult DeleteOne<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression)
    {
        if (document == null) throw new ArgumentNullException("null document");
        return _database.GetCollection<TDocument>(document.GetType().Name).DeleteOne<TDocument>(whereExpression);
    }

    /// <summary>
    /// 删除多个
    /// </summary>
    /// <param name="document"></param>
    /// <param name="whereExpression"></param>
    /// <typeparam name="TDocument"></typeparam>
    /// <returns></returns>
    /// <exception cref="ArgumentNullException"></exception>
    public DeleteResult DeleteMany<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression)
    {
        if (document == null) throw new ArgumentNullException("null document");
        return _database.GetCollection<TDocument>(document.GetType().Name).DeleteMany<TDocument>(whereExpression);
    }
    
    /// <summary>
    /// 查询扩展 
    /// </summary>
    /// <param name="whereExpression"></param>
    /// <returns></returns>
    public IMongoQueryable<TDocument> Queryable<TDocument>(TDocument document, Expression<Func<TDocument, bool>> whereExpression)
    {
        if (document == null) throw new ArgumentNullException("null document");
        return _database.GetCollection<TDocument>(document.GetType().Name).AsQueryable().Where(whereExpression);
    }
}