﻿using System.Text;
using BPA.Component.RabbitMQClient.Config;
using BPA.Component.RabbitMQClient.Options;
using BPA.Component.RabbitMQClient.Provider;
using Microsoft.Extensions.Logging;
using RabbitMQ.Client;
using RabbitMQ.Client.Events;

namespace BPA.Component.RabbitMQClient.Base;

/// <summary>
/// BaseConsumer,必须重写父类的Dequeue,以便接受消息
/// </summary>
/// <typeparam name="TMsgBody"></typeparam>
/// <typeparam name="TConsumer"></typeparam>
public abstract class BaseConsumer<TMsgBody, TConsumer> : IDisposable
{
    /// <summary>
    /// logger
    /// </summary>
    public readonly ILogger<TConsumer> Logger;

    /// <summary>
    /// Config
    /// </summary>
    internal RabbitMqQueueConfig Config;

    /// <summary>
    /// 消费者ID
    /// </summary>
    public string ConsumeId { private set; get; }

    /// <summary>
    /// BaseConsumer
    /// </summary>
    /// <param name="logger"></param>
    public BaseConsumer(ILogger<TConsumer> logger)
    {
        this.Logger = logger;
    }

    /// <summary>
    /// 初始化
    /// </summary> 
    /// <param name="config"></param>
    internal void Init(RabbitMqQueueConfig config)
    {
        this.Config = config;
        InitConsumer();
        config.InitConsumer = InitConsumer;
    }

    /// <summary>
    /// 重新连接
    /// </summary>
    internal void ReStart() => Config.ReStart();

    /// <summary>
    /// 定义消费者
    /// </summary>
    private void InitConsumer()
    {
        if (Config.Channel == null)
            throw new Exception("请先 UseQueue");
        //初始化消费者
        var consumer = new EventingBasicConsumer(Config.Channel);
        consumer.Received += (model, ea) =>
        {
            TMsgBody msgBody = default;
            var json = Encoding.UTF8.GetString(ea.Body.ToArray());
            try
            {
                msgBody = Newtonsoft.Json.JsonConvert.DeserializeObject<TMsgBody>(json, RabbitMQProvider.JsonSerializerSettings);
            }
            catch (Exception ex)
            {
                Logger.LogError(ex, "消息队列{queueName}消费消息失败:{message}", Config.Option.QueueName, json);
                if (Config.Option.IsAutoAck)
                    Config.Channel.BasicAck(ea.DeliveryTag, false);
                return;
            }

            if (Config.Option.IsAutoAck)
            {
                try
                {
                    Dequeue(msgBody, ea);
                }
                catch (Exception ex)
                {
                    Logger.LogError(ex, "消息队列{queueName}消费消息失败:{message}", Config.Option.QueueName, json);
                    Config.Channel.BasicAck(ea.DeliveryTag, false);
                }
            }
            else
            {
                try
                {
                    var result = Dequeue(msgBody, ea);
                    if (Config.Channel.IsClosed) return;
                    if (result) //消息处理成功,ack提交
                        Config.Channel.BasicAck(ea.DeliveryTag, false);
                    else //消息处理失败，把消息重新放入队列
                        Config.Channel.BasicNack(ea.DeliveryTag, false, true);
                }
                catch (Exception ex)
                {
                    Logger.LogError(ex, "消息队列{queueName}消费消息失败:{message}", Config.Option.QueueName, json);
                    if (Config.Channel.IsClosed) return;
                    //消息处理失败，把消息重新放入队列
                    Config.Channel.BasicNack(ea.DeliveryTag, false, true);
                }
            }
        };
        Console.WriteLine();
        Console.WriteLine($"*bpa.rabbitmqclient* 初始化消费者 : start ---------------------------");
        Console.WriteLine($"*bpa.rabbitmqclient* [{nameof(Config.Option.QueueName)}]={Config.Option.QueueName}");
        Console.WriteLine($"*bpa.rabbitmqclient* [FullName]={this.GetType().FullName}");
        Console.WriteLine($"*bpa.rabbitmqclient* [HashCode]={this.GetHashCode()}");
        Console.WriteLine($"*bpa.rabbitmqclient* [ConsumerID]={ConsumeId}");
        Console.WriteLine($"*bpa.rabbitmqclient* [Exchange]={Config.Option.ExchangeName}");
        Console.WriteLine($"*bpa.rabbitmqclient* 初始化消费者 : end ---------------------------");
        ConsumeId = Config.Channel.BasicConsume(queue: Config.Option.QueueName, autoAck: Config.Option.IsAutoAck, consumer: consumer);
    }

    /// <summary>
    /// 消息出列,消息接受者处理成功或者失败后返回true | false,
    /// 如果消息AutoAck=false(<see cref="QueueOption.IsAutoAck"/>)  ,那么当返回true是，提交ack，返回false是消息重新入列
    /// </summary>
    /// <param name="msgBody">消息内容</param> 
    /// <param name=" eventArgs">原始事件信息</param>
    public abstract bool Dequeue(TMsgBody msgBody, BasicDeliverEventArgs eventArgs);

    /// <summary>
    /// 释放资源
    /// </summary>
    public void Dispose()
    {
        Config.Channel?.Close();
        Config.Channel?.Dispose();
    }

    /// <summary>
    /// 关闭通道
    /// </summary>
    internal void ColseChannel()
    {
        Config.Channel?.Close();
    }

    /// <summary>
    /// 判断通道是否关闭
    /// </summary>
    internal bool ChannelIsColsed => Config.Channel == null || Config.Channel.IsClosed;
}