﻿using System.Text;
using BPA.Component.RabbitMQClient.Config;
using BPA.Component.RabbitMQClient.Enums;
using BPA.Component.RabbitMQClient.Provider;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using RabbitMQ.Client;
using ExchangeType = BPA.Component.RabbitMQClient.Enums.ExchangeType;

namespace BPA.Component.RabbitMQClient.Base
{
    /// <summary>
    /// 消息生产者基类
    /// </summary>
    /// <typeparam name="TMsgBody">消息体</typeparam>
    /// <typeparam name="TProducer">消息体</typeparam>
    public abstract class BaseProducer<TMsgBody, TProducer>
    {
        /// <summary>
        /// logger
        /// </summary>
        public readonly ILogger<TProducer> Logger;


        /// <summary>
        /// sender
        /// </summary>
        internal Action<TMsgBody, string, long?> Sender;

        /// <summary>
        /// BaseProducer
        /// </summary>
        /// <param name="logger"></param>
        public BaseProducer(ILogger<TProducer> logger) => this.Logger = logger;

        /// <summary>
        /// 初始化
        /// </summary> 
        /// <param name="config">队列配置信息</param>
        internal void Init(RabbitMqExchangeConfig config)
        {
            Sender = (msgBody, routingKey, expiration) =>
            {
                routingKey ??= string.Empty;
                var properties = config.Channel.CreateBasicProperties();
                //需要注意的是BasicProperties.deliveryMode，Non-persistent (1) or persistent (2) 这里指的是消息的持久化，
                properties.DeliveryMode = (byte) config.Option.DeliveryMode;
                properties.Persistent = config.Option.DeliveryMode == DeliveryModeType.Persistent;
                if (config.Option.Type == ExchangeType.X_Delayed_Message && expiration.HasValue)
                {
                    (properties.Headers ??= new Dictionary<string, object>()).Add("x-delay", expiration.Value.ToString());
                }
                else if (expiration.HasValue) //毫秒
                {
                    properties.Expiration = expiration.Value.ToString();
                }

                var json = JsonConvert.SerializeObject(msgBody, RabbitMQProvider.JsonSerializerSettings);
                var bodyBytes = Encoding.UTF8.GetBytes(json);
                config.Channel.BasicPublish(config.Option.Name, routingKey, properties, bodyBytes);
            };
        }

        /// <summary>
        /// 初始化,使用默认交换机
        /// </summary> 
        /// <param name="config">队列配置信息</param>
        internal void Init(RabbitMqQueueConfig config)
        {
            Sender = (msgBody, routingKey, expiration) =>
            {
                routingKey ??= string.Empty;
                var properties = config.Channel.CreateBasicProperties();
                //需要注意的是BasicProperties.deliveryMode，Non-persistent (1) or persistent (2) 这里指的是消息的持久化，
                properties.DeliveryMode = (byte) config.Option.DeliveryMode;
                properties.Persistent = config.Option.DeliveryMode == DeliveryModeType.Persistent;
                if (expiration.HasValue) //毫秒
                    properties.Expiration = expiration.Value.ToString();
                var json = JsonConvert.SerializeObject(msgBody, RabbitMQProvider.JsonSerializerSettings);
                var bodyBytes = Encoding.UTF8.GetBytes(json);
                //这里实际上使用的是默认交换机
                config.Channel.BasicPublish(config.Option.ExchangeName, config.Option.QueueName, properties, bodyBytes);
            };
        }


        /// <summary>
        /// 消息入列，如果子类重写了该,子类必须调用父类的Enqueue方法进行发送消息
        /// </summary> 
        /// <param name="msgBody">消息体</param>
        /// <param name="routingKey">路由key,自定义交换机有效,使用默认交换机时为当前队列名称</param>
        /// <param name="expiration"> 
        /// 延迟时间,默认为空, 单位 毫秒
        /// 使用延时时间时,消息到期后转到死信交换机,    
        /// 然后另外一个队列的消费者需要绑定死信交换机 
        /// </param>
        public virtual void Enqueue(TMsgBody msgBody, string routingKey = null, long? expiration = null)
        {
            Sender.Invoke(msgBody, routingKey, expiration);
        }
    }
}