﻿using BPA.Component.RabbitMQClient.Base;
using BPA.Component.RabbitMQClient.Connection;
using BPA.Component.RabbitMQClient.Provider;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.RabbitMQClient.Extensions
{
    /// <summary>
    /// RabbitMQ  扩展
    /// </summary>
    public static class RabbitMqConfigExtensions
    {

        /// <summary>
        /// 注入 <see cref="RabbitMQProvider"/> 到IOC 容器,单例; 
        /// <code> 
        /// 如果需要连接多个rabbit进行操作,请参照 
        /// UseRabbitMQProvider TRabbitMQProvider
        /// </code>
        /// 
        /// 
        /// </summary>
        /// <param name="service"></param>
        /// <param name="configFunc"></param>
        /// <returns></returns>
        public static IServiceCollection AddRabbitMQProvider(this IServiceCollection service, Func<IServiceProvider, RabbitMqConnectionConfig> configFunc)
        {
            if (configFunc == null)
                throw new ArgumentNullException(nameof(configFunc));
            service.AddSingleton(sp => new RabbitMQProvider(sp, configFunc(sp)));
            return service;
        }

        /// <summary>
        /// 注入自定义RabbitMQProvider  到IOC 容器,单例,TRabbitMQProvider 需要继承<see cref="RabbitMQProvider"/>
        /// </summary>
        /// <typeparam name="TRabbitMQProvider"></typeparam>
        /// <param name="service"></param>
        /// <param name="buildRabbitMQProvider"></param>
        /// <returns></returns>
        public static IServiceCollection AddRabbitMQProvider<TRabbitMQProvider>(this IServiceCollection service, Func<IServiceProvider, TRabbitMQProvider> buildRabbitMQProvider) where TRabbitMQProvider : RabbitMQProvider, new()
        {
            service.AddSingleton(sp => buildRabbitMQProvider(sp));
            return service;
        }

        /// <summary>
        /// 注入 生产者 到IOC 容器,单例 
        /// </summary>
        /// <typeparam name="TProducer"></typeparam>
        /// <typeparam name="TMsgBody"></typeparam>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddRabbitMQProducer<TProducer, TMsgBody>(this IServiceCollection services) where TProducer : BaseProducer<TMsgBody, TProducer>
        {
            services.AddSingleton<TProducer>();
            return services;
        }

        /// <summary>
        /// 注入 消息消费者 到IOC 容器,多例
        /// </summary>
        /// <typeparam name="TConsumer"></typeparam>
        /// <typeparam name="TMsgBody"></typeparam>
        /// <param name="services"></param>
        /// <returns></returns>
        public static IServiceCollection AddRabbitMQConsumer<TConsumer, TMsgBody>(this IServiceCollection services) where TConsumer : BaseConsumer<TMsgBody, TConsumer>
        {
            services.AddTransient<TConsumer>();
            return services;
        }

        /// <summary>
        /// 启动rabbitmq
        /// </summary>
        /// <param name="serviceProvider"></param>
        /// <returns></returns>
        public static TRabbitMQProvider UseRabbitMQProvider<TRabbitMQProvider>(this IServiceProvider serviceProvider) where TRabbitMQProvider : RabbitMQProvider
        {
            return serviceProvider.GetService<TRabbitMQProvider>();
        }



    }
}
