﻿using System;
using BPA.Component.RabbitMQClient.Base;
using BPA.Component.RabbitMQClient.Config;
using BPA.Component.RabbitMQClient.Options;
using BPA.Component.RabbitMQClient.Result;

namespace BPA.Component.RabbitMQClient.Extensions
{
    /// <summary>
    /// 交换机配置扩展
    /// </summary>
    public static class RabbitMqQueueConfigExtension
    {
        /// <summary>
        /// 配置队列选项
        /// </summary> 
        /// <param name="config"></param>
        /// <param name="configAction"></param>
        /// <returns></returns>
        public static RabbitMqQueueConfig ConfigOption(this RabbitMqQueueConfig config, Action<QueueOption> configAction)
        {
            if (configAction == null) return config;
            configAction.Invoke(config.Option);
            return config;
        }

        /// <summary>
        /// 使用队列
        /// </summary> 
        /// <param name="config"></param>
        /// <returns></returns>
        public static RabbitMqQueueConfig UseQueue(this RabbitMqQueueConfig config)
        {
            config.InitQueue();
            return config;
        }

        /// <summary>
        /// 启用生产者；消息-->默认交换机-->消息队列-->消费者
        /// </summary>
        /// <typeparam name="TProducer"></typeparam>
        /// <typeparam name="TMsgBody"></typeparam>
        /// <param name="config"></param>
        /// <exception cref="ArgumentNullException"></exception>
        /// <returns></returns>
        public static IRegisterProducerResult UseProducer<TMsgBody, TProducer>(this RabbitMqQueueConfig config)
                           where TProducer : BaseProducer<TMsgBody, TProducer>
        {
            if (config == null)
                throw new ArgumentNullException(nameof(config));
            var producer = config.RabbitMQProvider.ServiceProvider.GetService(typeof(TProducer)) as BaseProducer<TMsgBody, TProducer>;
            if (producer == null)
                throw new ArgumentNullException("请检查您的生产者是否注入的IOC容器,请调用 UseRabbitMqProducer 方法进行注入");
            if (producer.Sender != null)
                throw new ArgumentException($"当前生产者[{producer.GetType().FullName}]已经注册过了，不能再注册了");
            producer.Init(config);
            return new RegisterProducerResult<TMsgBody, TProducer>(producer);
        }

        /// <summary>
        /// 启用消费者,单个单个启用,可以多次调用
        /// </summary>
        /// <typeparam name="TConsumer">消费者泛型</typeparam>
        /// <typeparam name="TMsgBody">消息体</typeparam>
        /// <param name="config"></param> 
        /// <returns></returns>
        public static IRegisterConsumerResult UseConsumer<TMsgBody, TConsumer>(this RabbitMqQueueConfig config)
            where TConsumer : BaseConsumer<TMsgBody, TConsumer>
        {
            if (config == null)
                throw new ArgumentNullException(nameof(config));
            var consumer = config.RabbitMQProvider.ServiceProvider.GetService(typeof(TConsumer)) as BaseConsumer<TMsgBody, TConsumer>;
            if (consumer == null)
                throw new ArgumentNullException("请检查您的消费者是否注入的IOC容器,请调用 UseRabbitMqConsumer 方法进行注入");
            consumer.Init(config);
            return new RegisterConsumerResult<TMsgBody, TConsumer>(consumer);
        }

        /// <summary>
        /// 启用消费者,批量启用
        /// </summary>
        /// <typeparam name="TConsumer">消费者泛型</typeparam>
        /// <typeparam name="TMsgBody">消息体</typeparam>
        /// <param name="config"></param>
        /// <param name="instanceCount"></param> 
        /// <returns></returns>
        public static List<IRegisterConsumerResult> UseMultipleConsumer<TMsgBody, TConsumer>(this RabbitMqQueueConfig config, int instanceCount = 1)
            where TConsumer : BaseConsumer<TMsgBody, TConsumer>
        {
            if (instanceCount < 1)
                instanceCount = 1;
            var results = new List<IRegisterConsumerResult>();
            for (var i = 0; i < instanceCount; i++)
                results.Add(config.UseConsumer<TMsgBody, TConsumer>());
            return results;
        }


    }
}
