﻿using BPA.Component.RabbitMQClient.Enums;

namespace BPA.Component.RabbitMQClient.Options
{
    /// <summary>
    /// 队列配置选项
    /// </summary>
    public class QueueOption
    {
        /// <summary>
        /// 队列名称
        /// </summary>
        public QueueName QueueName { set; get; }

        /// <summary>
        /// 交换机名称,交换机模式下需要配置交换机名称
        /// </summary>
        public ExchangeName ExchangeName { set; get; }

        /// <summary>
        /// 消息投递模式 ，默认持久化<see cref="DeliveryModeType.Persistent"/>,  是否持久化消息, 
        /// </summary>
        public DeliveryModeType DeliveryMode { set; get; } = DeliveryModeType.Persistent;


        private string routingKey = string.Empty;
        /// <summary>
        /// 路由key,默认为string.Empty
        /// </summary>
        public string RoutingKey { set => routingKey = value ?? string.Empty; get => routingKey; }

        /// <summary>
        /// 是否Ack自动提交，默认false,消息处理失败后，需要重新放入队列
        /// </summary>
        public bool IsAutoAck { set; get; } = false;

        /// <summary>
        /// 队列是否持久,默认true
        /// RabbitMQ的队列和消息存放于内存中，在出现任何异常情况，导致RabbitMQ异常中止，内存中的数据将会丢失。若要在RabbitMQ从异常恢复后仍然保存有异常中止前的消息，就需将消息持久化，即将消息同时保存于磁盘中。RabbitMQ可将消息保存到Erlang自带的Mnesia数据库中，当RabbitMQ重启之后会读取该数据库。
        /// 声明队列的方法中，“durable”参数即指定队列是否是持久化的。使消息持久化，需要队列和消息都是持久化的，并且通常交换机也应该是持久化的。RabbitMQ的默认交换机“(AMQP default)”是持久化的，对于与其绑定的队列，将队列声明为持久化的队列，并发送持久化的消息，即可将消息持久化。
        /// </summary>
        public bool Durable { set; get; } = true;

        /// <summary>
        /// 独有的；排外的；专一的，默认false
        /// 对于排他队列，只有创建它的连接有权访问，连接断开后，排他队列将自动删除。这种队列适用于一个队列仅对应一个客户端收发消息的场景。在声明队列时，将exclusive参数设置为true即可声明一个排他队列。
        /// </summary>
        public bool Exclusive { set; get; } = false;

        /// <summary>
        /// 自动删除，默认false
        /// 若队列的autoDelete（自动删除）属性开启，当队列的最后一个消费者断开时，该队列会被自动删除。
        /// 声明一个队列，autoDelete参数设置为true，进入Web管理界面，可以看到，队列列表中出现了被创建的自动删除队列，Features列的“AD”即表明该队列是自动删除的。
        /// </summary>
        public bool AutoDelete { set; get; } = false;

        /// <summary>
        /// 设置消息过期时间,作用于整个队列
        /// </summary>
        /// <param name="millisecond"></param>
        /// <returns></returns>
        public QueueOption SetMeessgeTTL(long millisecond)
        {
            Arguments[X_MESSAGE_TTL] = millisecond;
            return this;
        }

        /// <summary>
        /// 设置死信交换机名称,当队列内消息到达过期时间后会推送消息到死信交换机
        /// </summary>
        /// <param name="exchangeName">交换机</param>
        /// <returns></returns>
        public QueueOption SetDeadLetterExchangeName(string exchangeName)
        {
            Arguments[X_DEAD_LETTER_EXCHANGE] = exchangeName;
            return this;
        }

        /// <summary>
        /// 设置死信交换机的 路由Key
        /// </summary>
        /// <param name="routeKey">路由key</param>
        /// <returns></returns>
        public QueueOption SetDeadLetterRoutingKey(string routeKey)
        {
            Arguments[X_DEAD_LETTER_ROUTING_KEY] = routeKey;
            return this;
        }

        /// <summary>
        /// 消息生存时间，单位毫秒 value =long
        /// </summary>
        public const string X_MESSAGE_TTL = "x-message-ttl";

        /// <summary>
        /// 队列闲置时间，单位毫秒 value =long
        /// </summary>
        public const string X_EXPIRES = "x-expires";

        /// <summary>
        /// 队列最大长度 value =long
        /// </summary>
        public const string X_MAX_LENGTH = "x-max-length";

        /// <summary>
        /// 队列最大占用空间，单位字节  value =long
        /// </summary>
        public const string X_MAX_LENGTH_BYTES = "x-max-length-bytes";

        /// <summary>
        /// 移出的消息发送至 某一个 交换机 value =string
        /// </summary>
        public const string X_DEAD_LETTER_EXCHANGE = "x-dead-letter-exchange";

        /// <summary>
        /// 移出的消息发送至某一个交换机，路由键为 的名字 value =string
        /// </summary>
        public const string X_DEAD_LETTER_ROUTING_KEY = "x-dead-letter-routing-key";

        /// <summary>
        /// 最大优先级为10 value =long
        /// </summary>
        public const string X_MAX_PRIORITY = "x-max-priority";

        /// <summary>
        /// 队列模式  value =lazy(延迟模式)
        /// </summary>
        public const string X_QUEUE_MODE = "x-queue-mode";


        /// <summary>
        /// Arguments,队列扩展参数
        /// arguments.put("x-message-ttl", 10000);		// 消息生存时间，10s
        /// arguments.put("x-expires", 20000);			// 队列闲置时间，20s
        /// arguments.put("x-max-length", 100);			// 队列最大长度，100条消息
        /// arguments.put("x-max-length-bytes", 1024);	// 队列最大占用空间，1024字节
        /// arguments.put("x-dead-letter-exchange", "amq.direct");	//  移出的消息发送至“amq.direct”交换机，路由键为“dead”
        /// arguments.put("x-dead-letter-routing-key", "dead");
        /// arguments.put("x-max-priority", 10);			// 最大优先级为10
        /// arguments.put("x-queue-mode", "lazy");		// 延迟模式 
        /// </summary>
        public IDictionary<string, object> Arguments { set; get; } = new Dictionary<string, object>();



    }
}
