﻿using BPA.Component.RabbitMQClient.Config;
using BPA.Component.RabbitMQClient.Connection;
using BPA.Component.RabbitMQClient.Enums;
using BPA.Component.RabbitMQClient.Options;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using RabbitMQ.Client;
using ExchangeType = BPA.Component.RabbitMQClient.Enums.ExchangeType;

namespace BPA.Component.RabbitMQClient.Provider
{
    /// <summary>
    /// 当恢复连接时触发
    /// </summary>
    internal delegate void OnRecoveryConnect();

    /// <summary>
    /// RabbitMQ 提供者
    /// </summary>
    public class RabbitMQProvider
    {
        internal static readonly JsonSerializerSettings JsonSerializerSettings;

        static RabbitMQProvider()
        {
            JsonSerializerSettings = new JsonSerializerSettings
            {
                DateFormatString = "yyyy-MM-dd HH:mm:ss:fff",
                ReferenceLoopHandling = Newtonsoft.Json.ReferenceLoopHandling.Ignore,
            };
            JsonSerializerSettings.Converters.Add(new StringEnumConverter());
        }

        /// <summary>
        /// Connection
        /// </summary>
        internal IConnection Connection;

        /// <summary>
        /// serviceProvider
        /// </summary>
        internal readonly IServiceProvider ServiceProvider;

        /// <summary>
        /// 当恢复连接时触发
        /// </summary>
        internal event OnRecoveryConnect OnRecoveryConnect;

        /// <summary>
        /// 是否重连
        /// </summary>
        private bool isRecoveryConnect = false;

        /// <summary>
        /// MQConfig配置
        /// </summary>
        public readonly RabbitMqConnectionConfig MQConfig;

        /// <summary>
        /// connection
        /// </summary>
        /// <param name="serviceProvider"></param> 
        ///<param name=" mqConfig">工厂</param>
        public RabbitMQProvider(IServiceProvider serviceProvider, RabbitMqConnectionConfig mqConfig)
        {
            ServiceProvider = serviceProvider;
            MQConfig = mqConfig;
            BuildConnectionFactory();
        }

        /// <summary>
        /// BuildConnectionFactory
        /// </summary> 
        /// <returns></returns>
        private void BuildConnectionFactory()
        {
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: start ---------------------------");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: [HostName]={ MQConfig.HostName }");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: [Port]={ MQConfig.Port}");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: [Password]={ MQConfig.Password}");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: [UserName]={ MQConfig.UserName}");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: [VirtualHost]={ MQConfig.VirtualHost}");
            Console.WriteLine($"*bpa.rabbitmqclient* build connetion info: end ---------------------------");
            var factory = new ConnectionFactory()
            {
                HostName = MQConfig.HostName,
                Port = MQConfig.Port,
                Password = MQConfig.Password,
                UserName = MQConfig.UserName,
                VirtualHost = MQConfig.VirtualHost,
                AutomaticRecoveryEnabled = true,
                RequestedHeartbeat = TimeSpan.FromSeconds(30),
                TopologyRecoveryEnabled = true,
                NetworkRecoveryInterval = TimeSpan.FromSeconds(10)
            };

            var connection = factory.CreateConnection();
            connection.ConnectionShutdown += OnConnectionShutdown;
            Connection = connection;
            if (!isRecoveryConnect)
                return;
            try
            {
                OnRecoveryConnect();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }

        }

        /// <summary>
        /// 连接中断
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnConnectionShutdown(object sender, ShutdownEventArgs e)
        {
            isRecoveryConnect = true;
            Console.WriteLine($"*bpa.rabbitmqclient* connection shutdown info: {e.MethodId} ");
        }

        /// <summary>
        /// 构建一个自定义交换机配置对象
        /// </summary> 
        /// <param name="exchangeName">交换机</param>
        /// <param name="type"></param> 
        /// <returns></returns>
        public RabbitMqExchangeConfig BuildExchangeConfig(ExchangeName exchangeName, ExchangeType type = ExchangeType.Direct)
        {
            if (exchangeName == null || string.IsNullOrEmpty(exchangeName))
                throw new ArgumentNullException(nameof(exchangeName));
            return BuildExchangeConfig(exchangeName, (option) => option.Type = type);
        }

        /// <summary>
        /// 构建一个自定义交换机配置对象
        /// </summary> 
        /// <param name="exchangeName">交换机</param>
        /// <param name="optionAction"></param> 
        /// <returns></returns>
        public RabbitMqExchangeConfig BuildExchangeConfig(ExchangeName exchangeName, Action<ExchangeOption> optionAction)
        {
            if (exchangeName == null || string.IsNullOrEmpty(exchangeName))
                throw new ArgumentNullException(nameof(exchangeName));
            var option = new ExchangeOption
            {
                Name = exchangeName,
                Type = ExchangeType.Fanout,
                DeliveryMode = DeliveryModeType.Persistent,
                AutoDelete = false,
                Durable = true,
                RoutingKey = string.Empty
            };
            optionAction?.Invoke(option);
            return new RabbitMqExchangeConfig(this, option);
        }

        /// <summary>
        /// 构建一个队列配置对象,使用自定义交换机
        /// </summary>
        /// <param name="queueName">队列名称</param> 
        /// <param name="exchangeName">自定义交换机</param> 
        /// <returns></returns>
        public RabbitMqQueueConfig BuildQueueConfig(QueueName queueName, ExchangeName exchangeName)
        {
            return BuildQueueConfig(queueName, exchangeName, _ => { });
        }

        /// <summary>
        /// 构建一个队列配置对象,使用自定义交换机
        /// </summary>
        /// <param name="queueName">队列名称</param> 
        /// <param name="exchangeName">交换机</param>
        /// <param name="optionAction"></param> 
        /// <returns></returns>
        public RabbitMqQueueConfig BuildQueueConfig(QueueName queueName, ExchangeName exchangeName, Action<QueueOption> optionAction)
        {
            if (queueName == null || string.IsNullOrEmpty(queueName))
                throw new ArgumentNullException(nameof(queueName));
            if (exchangeName == null || string.IsNullOrEmpty(exchangeName))
                throw new ArgumentNullException(nameof(exchangeName));
            var option = new QueueOption
            {
                QueueName = queueName,
                ExchangeName = exchangeName,
                Durable = true,
                RoutingKey = string.Empty
            };
            optionAction?.Invoke(option);
            return new RabbitMqQueueConfig(this, option);
        }

        /// <summary>
        /// 构建一个队列配置对象,使用默认交换机
        /// </summary>
        /// <param name="queueName">队列名称</param>
        /// <param name="optionAction"></param>  
        /// <returns></returns>
        public RabbitMqQueueConfig BuildQueueConfig(QueueName queueName, Action<QueueOption> optionAction)
        {
            var option = new QueueOption
            {
                QueueName = queueName,
                ExchangeName = ExchangeName.Build(string.Empty),
                Durable = true,
                RoutingKey = string.Empty
            };
            optionAction?.Invoke(option);
            return new RabbitMqQueueConfig(this, option);
        }

        /// <summary>
        /// 构建一个队列配置对象,使用默认交换机
        /// </summary>
        /// <param name="queueName">队列名称</param> 
        /// <returns></returns>
        public RabbitMqQueueConfig BuildQueueConfig(QueueName queueName)
        {
            return BuildQueueConfig(queueName, _ => { });
        }
    }
}
