﻿using BPA.Component.ApolloClient;
using BPA.Component.LogClient.Extensions;
using BPA.Component.RabbitMQClient.Enums;
using BPA.Component.RabbitMQClient.Extensions;
using BPA.Component.RabbitMQClient.Provider;
using BPA.Component.RabbitMQClientTester.Custom;
using BPA.Component.RabbitMQClientTester.Default;
using BPA.Component.RabbitMQClientTester.Delays;
using BPA.Component.RabbitMQClientTester.Modesl;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;

namespace BPA.Component.RabbitMQClientTester
{
    internal static class Program
    {
        private static void Main(string[] args)
        {
            DelayExchange();
            Console.ReadLine();
        }

        /// <summary>
        /// 默认交换机
        /// </summary>
        private static void DefaultExchange()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();

            var configuration = builder.Build();
            configuration.AddApolloConfiguration();

            var services = new ServiceCollection();
            services.AddBPALog();
            services.AddApollo<TestConfig>();
            services.AddRabbitMQProvider(provider => provider.GetService<TestConfig>()?.RabbitMqConfig);
            services.AddRabbitMQProducer<TestDefaultQueueProducer, TestMsgBody>();
            services.AddRabbitMQConsumer<TestDefaultQueueConsumer1, TestMsgBody>();

            var serviceProvider = services.BuildServiceProvider();
            var rabbitMqProvider = serviceProvider.UseRabbitMQProvider<RabbitMQProvider>();

            var queue1 = rabbitMqProvider.BuildQueueConfig(MqNameConfig.TestDefaultQueue1);
            queue1.ConfigOption(_ => { });
            queue1.UseQueue();
            queue1.UseProducer<TestMsgBody, TestDefaultQueueProducer>();

            var producer = serviceProvider.GetService<TestDefaultQueueProducer>();
            if (producer == null) throw new ArgumentNullException();
            for (var i = 0; i <= 20; i++)
            {
                producer.Enqueue(new TestMsgBody {Date = DateTime.Now, Id = i});
            }

            queue1.UseConsumer<TestMsgBody, TestDefaultQueueConsumer1>(); //启用一个消费者
            queue1.UseMultipleConsumer<TestMsgBody, TestDefaultQueueConsumer1>(3); //启用多个消费者

            var id = 0;
            while (true)
            {
                //生产消息
                producer.Enqueue(new TestMsgBody {Date = DateTime.Now, Id = id++});
                Console.ReadLine();
                Thread.Sleep(1000);
            }
        }

        /// <summary>
        /// 测试自定义交换机
        /// </summary>
        private static void CustomExchange()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();

            var configuration = builder.Build();
            configuration.AddApolloConfiguration();

            var services = new ServiceCollection();
            services.AddBPALog();
            services.AddApollo<TestConfig>();
            services.AddRabbitMQProvider(provider => provider.GetService<TestConfig>()?.RabbitMqConfig);
            services.AddRabbitMQProducer<TestCustomQueueProducer, TestMsgBody>();
            services.AddRabbitMQConsumer<TestCustomQueueConsumer1, TestMsgBody>();
            services.AddRabbitMQConsumer<TestCustomQueueConsumer2, TestMsgBody>();

            var serviceProvider = services.BuildServiceProvider();
            var rabbitMqProvider = serviceProvider.UseRabbitMQProvider<RabbitMQProvider>();

            var exchangeConfig = rabbitMqProvider.BuildExchangeConfig(MqNameConfig.TestCustomExchange);
            exchangeConfig.UseExchange();
            exchangeConfig.UseProducer<TestMsgBody, TestCustomQueueProducer>();

            var queue1 = rabbitMqProvider.BuildQueueConfig(MqNameConfig.TestCustomQueue1, MqNameConfig.TestCustomExchange);
            queue1.ConfigOption(_ => { });
            queue1.UseQueue();
            queue1.UseConsumer<TestMsgBody, TestCustomQueueConsumer1>();
            queue1.UseConsumer<TestMsgBody, TestCustomQueueConsumer1>();
            queue1.UseMultipleConsumer<TestMsgBody, TestCustomQueueConsumer1>(3);

            var queue2 = rabbitMqProvider.BuildQueueConfig(MqNameConfig.TestCustomQueue2, MqNameConfig.TestCustomExchange);
            queue2.ConfigOption(_ => { });
            queue2.UseQueue();
            queue2.UseConsumer<TestMsgBody, TestCustomQueueConsumer2>();
            queue2.UseMultipleConsumer<TestMsgBody, TestCustomQueueConsumer2>(3);

            var id = 0;
            while (true)
            {
                var producer = serviceProvider.GetService<TestCustomQueueProducer>();
                producer.Enqueue(new TestMsgBody {Date = DateTime.Now, Id = id++});
                Thread.Sleep(1000);
            }
        }

        /// <summary>
        /// 测试延时
        /// </summary>
        private static void DelayExchange()
        {
            var builder = new ConfigurationBuilder();
            builder.AddEnvironmentVariables();

            var configuration = builder.Build();
            configuration.AddApolloConfiguration();

            var services = new ServiceCollection();
            services.AddBPALog();
            services.AddApollo<TestConfig>();
            services.AddRabbitMQProvider(provider => provider.GetService<TestConfig>()?.RabbitMqConfig);
            services.AddRabbitMQProducer<TestDelayQueueProducer, TestMsgBody>();
            services.AddRabbitMQConsumer<TestDelayQueueConsumer, TestMsgBody>();

            var serviceProvider = services.BuildServiceProvider();
            var rabbitMqProvider = serviceProvider.UseRabbitMQProvider<RabbitMQProvider>();

            var exchangeConfig1 = rabbitMqProvider.BuildExchangeConfig(MqNameConfig.TestDelayExchange1, ExchangeType.X_Delayed_Message);
            exchangeConfig1.UseExchange();
            exchangeConfig1.UseProducer<TestMsgBody, TestDelayQueueProducer>();
            
            //场景1：消息--发送到(延时消息)--交换机1---队列1---消费者
            //场景2：消息--发送到(延时消息)--交换机1---队列1---过期(延时)---交换机2---队列2---消费者
            rabbitMqProvider.BuildExchangeConfig(MqNameConfig.TestDelayExchange2).UseExchange();

            var queue1 = rabbitMqProvider.BuildQueueConfig(MqNameConfig.TestDelayQueue1, MqNameConfig.TestDelayExchange1);
            queue1.UseQueue();
            queue1.UseConsumer<TestMsgBody, TestDelayQueueConsumer>();
            
            var id = 0;
            while (true)
            {
                var producer = serviceProvider.GetService<TestDelayQueueProducer>();
                var delay = new Random().Next(1, 15) * 1000L;
                producer.Enqueue(new TestMsgBody {Date = DateTime.Now, Id = id++, Message = $"延时{delay}"}, null, delay);
                Console.ReadLine();
            }
        }
    }
}