﻿using CSRedis;
using Microsoft.Extensions.DependencyInjection;

namespace BPA.Component.RedisClient;

/// <summary>
/// rediscore 注入扩展
/// </summary>
public static class RedisConfigExtensions
{
    /// <summary>
    /// lockObject
    /// </summary>
    private static readonly object _lockObject = new();

    /// <summary>
    /// 单例连接客户端
    /// </summary>
    private static CSRedisClient CSRedisClientSingleton;

    /// <summary>
    /// 当单例redis连接字符串改变时，调用该方法会会根据新的连接字符串，生成新的连接字符串重新生成新的client
    /// </summary>
    /// <param name="connectionString"></param>
    public static void OnRedisSingletonConnectionStringChange(string connectionString) =>
        CreateRedisSingleton(connectionString, true);

    /// <summary>
    /// redis client 注入单例, 如果项目中有且只需要连接一个redis集群时 可使用该方法注入,
    /// 使用时从容器中获取 <see cref="CSRedisClient"/>(强烈建议,有扩展)或<see cref="RedisHelper"/>
    /// </summary>
    /// <param name="services"></param>
    /// <param name="getConnectionStringFuc">该fun需要返会的是redis连接字符串</param>
    /// <returns></returns>
    public static IServiceCollection UseRedisSingleton(this IServiceCollection services,
        Func<IServiceProvider, string> getConnectionStringFuc)
    {
        services.AddSingleton(serviceProvider => CreateRedisSingleton(getConnectionStringFuc(serviceProvider)));
        return services;
    }

    /// <summary>
    /// 注入redis工厂,适用于多个redis集群,或连接多个不同的redis,
    /// 当前工厂会被注入为单例,工厂内部应实现 连接字符串改变时触发的事件或者方法，当连接字符改变时，需要重新初始化对应的client,
    /// 使用工厂时，直接从容器中获取工厂对象TRedisClientFactory
    /// </summary>
    /// <param name="services"></param> 
    /// <returns></returns>
    public static IServiceCollection UseRedisFactory<TRedisClientFactory>(this IServiceCollection services)
        where TRedisClientFactory : RedisClientFactory
    {
        services.AddSingleton<TRedisClientFactory>();
        return services;
    }

    /// <summary>
    /// 创建单例连接对象
    /// </summary>
    /// <param name="connectionString">连接字符串</param>
    /// <param name="isFocesCreate">是否强制创建</param>
    /// <returns></returns>
    public static CSRedisClient CreateRedisSingleton(string connectionString, bool isFocesCreate = false)
    {
        if (isFocesCreate)
        {
            lock (_lockObject)
            {
                CSRedisClientSingleton = new CSRedisClient(connectionString);
                RedisHelper.Initialization(CSRedisClientSingleton);
            }
        }
        else if (CSRedisClientSingleton == null)
        {
            lock (_lockObject)
            {
                if (CSRedisClientSingleton == null)
                {
                    CSRedisClientSingleton = new CSRedisClient(connectionString);
                    RedisHelper.Initialization(CSRedisClientSingleton);
                }
            }
        }

        return CSRedisClientSingleton;
    }
}