﻿using System.Net;
using System.Text;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.DTOCommon.Langs;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace BPA.Component.SDKCommon
{
    /// <summary>
    /// SDKHttpMessageHandler
    /// </summary>
    internal class SDKHttpMessageHandler : DelegatingHandler
    {
        protected readonly ILogger<SDKHttpMessageHandler> Logger;
        protected readonly SDKUrlFuncCollection SDKUrlFuncCollection;
        protected readonly IServiceProvider ServiceProvider;

        public SDKHttpMessageHandler(
            IServiceProvider serviceProvider,
            ILogger<SDKHttpMessageHandler> logger,
            SDKUrlFuncCollection sdkUrlFuncCollection)
        {
            ServiceProvider = serviceProvider;
            Logger = logger;
            SDKUrlFuncCollection = sdkUrlFuncCollection;
        }

        /// <summary>
        /// 发送请求
        /// </summary>
        /// <param name="request"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        protected override async Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            var log = SDKExtensions.IsDebug ? new StringBuilder() : null;
            try
            {
                if (SDKExtensions.IsDebug)
                {
                    ReadReqArgs(request, log);
                }

                var responseMessage = await base.SendAsync(request, cancellationToken);
                if (responseMessage.StatusCode == HttpStatusCode.OK)
                {
                    if (SDKExtensions.IsDebug)
                    {
                        ReadRspArgs(responseMessage, log);
                    }
                    return responseMessage;
                }

                throw new Exception($"请求失败:StatusCode:{(int) (responseMessage.StatusCode)}-{responseMessage.StatusCode}");
            }
            catch (Exception ex)
            {
                if (log == null)
                {
                    log = new StringBuilder();
                    ReadReqArgs(request, log);
                }

                Logger.LogError(ex, log.ToString());
                log = null;

                var langType = SDKUrlFuncCollection.GetClientLangFunc == null
                    ? LangType.Zn
                    : SDKUrlFuncCollection.GetClientLangFunc(ServiceProvider);

                var result = BaseResult.Build(ConstLangPackge.Error, "请稍后重试", "please try again later");

                result.Render(langType, request);

                return new HttpResponseMessage
                {
                    Content = new StringContent(JsonConvert.SerializeObject(result,
                        BPANewtonsoftJsonContentSerializer.JsonSerializerSettings))
                };
            }
            finally
            {
                if (log != null)
                    Logger.LogInformation(log.ToString());
            }
        }

        /// <summary>
        /// 读取请求参数
        /// </summary>
        /// <param name="request"></param>
        /// <param name="stringBuilder"></param>
        private static void ReadReqArgs(HttpRequestMessage request, StringBuilder? stringBuilder)
        {
            stringBuilder ??= new StringBuilder();
            stringBuilder.Append($"【reqUrl】【{DateTime.Now}】: \n\t{request.RequestUri}\n");

            if (request.Headers.Count() > 0)
            {
                stringBuilder.Append(
                    $"Headers: {string.Join("\n\t", request.Headers.Select(x => $"{x.Key}={x.Value?.FirstOrDefault()}"))} \n");
            }

            if (request.Content == null) return;

            if (request.Content is not MultipartFormDataContent)
            {
                var isJsonArgs = request.Content.Headers.ContentType?.MediaType?.Contains("json", StringComparison.OrdinalIgnoreCase);
                if (isJsonArgs.HasValue && isJsonArgs.Value)
                {
                    var reqArgs = request.Content.ReadAsStringAsync().GetAwaiter().GetResult();
                    stringBuilder.Append($"\n【reqArgs】:\n\t{reqArgs}\n");
                }
            }
        }

        /// <summary>
        /// 读取返回值
        /// </summary>
        /// <param name="httpResponse"></param>
        /// <param name="stringBuilder"></param>
        private static void ReadRspArgs(HttpResponseMessage httpResponse, StringBuilder? stringBuilder)
        {
            if (httpResponse.Content.Headers.ContentLength <= 0)
                return;
            var isJsonArgs = httpResponse.Content.Headers.ContentType?.MediaType?.Contains("json", StringComparison.OrdinalIgnoreCase);
            if (!isJsonArgs.HasValue || !isJsonArgs.Value)
                return;
            var rspArgs = httpResponse.Content.ReadAsStringAsync().GetAwaiter().GetResult();
            stringBuilder ??= new StringBuilder();
            stringBuilder.Append($"\n【rspArgs】【{DateTime.Now}】:\n\t{rspArgs}\n");
        }
    }

    /// <summary>
    /// SDKHttpMessageHandlerWithTrackId
    /// </summary>
    internal class SDKHttpMessageHandlerWithTrackId : SDKHttpMessageHandler
    {
        /// <summary>
        /// 跟踪ID head名称
        /// </summary>
        public const string TraceIdHeadName = "X-TraceId";

        protected readonly IHttpContextAccessor HttpContextAccessor;

        /// <summary>
        /// SDKHttpMessageHandlerWithTrackId
        /// </summary>
        /// <param name="serviceProvider"></param>
        /// <param name="logger"></param>
        /// <param name="sdkUrlFuncCollection"></param>
        /// <param name="httpContextAccessor"></param>
        public SDKHttpMessageHandlerWithTrackId(
            IServiceProvider serviceProvider,
            ILogger<SDKHttpMessageHandlerWithTrackId> logger,
            SDKUrlFuncCollection sdkUrlFuncCollection,
            IHttpContextAccessor httpContextAccessor)
            : base(serviceProvider, logger, sdkUrlFuncCollection)
        {
            HttpContextAccessor = httpContextAccessor;
        }

        /// <summary>
        /// override
        /// </summary>
        /// <param name="request"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        protected override Task<HttpResponseMessage> SendAsync(HttpRequestMessage request, CancellationToken cancellationToken)
        {
            request.Headers.Add(TraceIdHeadName, GetTrackId());
            return base.SendAsync(request, cancellationToken);
        }

        /// <summary>
        /// 获取trackId
        /// </summary>
        /// <returns></returns>
        private string GetTrackId()
        {
            var trackId = HttpContextAccessor.HttpContext?.TraceIdentifier;

            if (trackId != null)
            {
                Console.WriteLine($"HttpContext Id: {trackId}");
                return trackId;
            }

            trackId = System.Diagnostics.Activity.Current?.Id;

            return trackId ?? Guid.NewGuid().ToString("n");
        }
    }
}