﻿using System.Text;
using BPA.Component.DTOCommon.Langs;
using BPA.Component.Extensions;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc.Formatters;
using Microsoft.AspNetCore.WebUtilities;

namespace BPA.Component.WebApiExtensions.Extensions
{
    /// <summary>
    /// HttpContextExtension
    /// </summary>
    public static class HttpContextExtension
    {
        /// <summary>
        /// 客户端ip header key
        /// </summary>
        public static string ClientIPHeaderKey;

        /// <summary>
        /// 获取客户端请求IP 
        /// <code>
        /// Headers["HTTP_X_FORWARDED_FOR"]
        /// Headers["REMOTE_ADDR"]
        /// HttpContext.Connection.RemoteIpAddress.MapToIPv4()
        /// </code>
        /// </summary>
        /// <param name="httpRequest"></param>
        /// <returns></returns>
        public static string? GetClientIp(this HttpRequest httpRequest)
        {
            try
            {
                string? strIpAddress;
                if (ClientIPHeaderKey.IsNotEmpty())
                {
                    strIpAddress = httpRequest.Headers[ClientIPHeaderKey];
                    if (strIpAddress.IsNotNull())
                        goto end;
                }

                strIpAddress = httpRequest.Headers["x-Real-IP"];
                if (strIpAddress.IsNotNull())
                    goto end;

                strIpAddress = httpRequest.Headers["x-Original-Forwarded-For"];
                if (strIpAddress.IsNotNull())
                    goto end;

                strIpAddress = httpRequest.Headers["x-Forwarded-For"];
                if (strIpAddress.IsNotNull())
                    goto end;

                strIpAddress = httpRequest.HttpContext.Connection.RemoteIpAddress?.MapToIPv4().ToString();
                end:
                {
                    if (strIpAddress != null && strIpAddress.Contains(','))
                        strIpAddress = strIpAddress.Split(", ")[0];
                }

                return strIpAddress;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return string.Empty;
            }
        }

        /// <summary>
        /// 获取客户端语言类型 Headers["Accept-Language"]
        /// </summary>
        /// <param name="httpRequest"></param>
        /// <param name="defaultLang">默认中文</param>
        /// <returns></returns>
        public static LangType GetLanguage(this HttpRequest? httpRequest, LangType defaultLang = LangType.Zn)
        {
            if (httpRequest == null)
                return defaultLang;
            var acceptLanguage = httpRequest.Headers["Accept-Language"];
            return acceptLanguage.ToString().ToTrim().ToLower() == "en" ? LangType.En : LangType.Zn;
        }

        /// <summary>
        /// 获取请求地址
        /// </summary>
        /// <param name="request"></param> 
        /// <returns></returns>
        public static string GetRequestUrl(this HttpRequest request)
        {
            return $"{request.Scheme}://{request.Host}{request.Path}{request.QueryString}";
        }

        /// <summary>
        /// 读取请求参数
        /// </summary>
        /// <param name="request"></param>
        /// <param name="excludeFileForm">是否排除 当提交的数据中有 文件数据</param>
        /// <returns></returns>
        public static async Task<string?> ReadBodyAsync(this HttpRequest request, bool excludeFileForm = true)
        {
            try
            {
                // ReSharper disable once ConditionIsAlwaysTrueOrFalse
                if (request.ContentLength == 0 && request.Body == null)
                {
                    return null;
                }

                if (excludeFileForm && request.HasFormContentType && request.Form.Files.Count > 0)
                {
                    return null;
                }

                await EnableRewindAsync(request).ConfigureAwait(false);
                var encoding = GetRequestEncoding(request);
                return await ReadStreamAsync(request.Body, encoding).ConfigureAwait(false);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                return null;
            }
        }

        /// <summary>
        /// 获取请求编码
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        private static Encoding GetRequestEncoding(HttpRequest request)
        {
            var requestContentType = request.ContentType;
            var requestMediaType = requestContentType == null ? default(MediaType) : new MediaType(requestContentType);
            var requestEncoding = requestMediaType.Encoding ?? Encoding.UTF8;
            return requestEncoding;
        }

        /// <summary>
        /// 是否跳转
        /// </summary>
        /// <param name="request"></param>
        private static async Task EnableRewindAsync(HttpRequest request)
        {
            request.EnableBuffering();
            await request.Body.DrainAsync(CancellationToken.None);
            request.Body.Seek(0L, SeekOrigin.Begin);
        }

        private static async Task<string> ReadStreamAsync(Stream stream, Encoding encoding)
        {
            using var sr = new StreamReader(stream, encoding, true, 1024, true);
            var str = await sr.ReadToEndAsync();
            stream.Seek(0, SeekOrigin.Begin); //内容读取完成后需要将当前位置初始化，否则后面的InputFormatter会无法读取
            return str;
        }
    }
}