﻿using System.Runtime.CompilerServices;
using System.Text;
using AutoMapper;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.DTOCommon.JsonConverters;
using BPA.Component.DTOCommon.Langs;
using BPA.Component.Extensions;
using BPA.Component.WebApiExtensions.Filters;
using BPA.Component.WebApiExtensions.ValidationExtensions;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Diagnostics;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Microsoft.Extensions.Primitives;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;

namespace BPA.Component.WebApiExtensions.Extensions;

/// <summary>
/// mvc 配置项 扩展
/// </summary>
public static class WebApiExtension
{
    /// <summary>
    /// 跟踪ID head名称
    /// </summary>
    public const string TraceIdHeadName = "X-TraceId";

    /// <summary>
    ///  x-Request-ID 跟踪ID head名称
    /// </summary>
    public const string RequestIdHeadName = "x-Request-ID";


    // /// <summary>
    // /// 注入AutoMapper
    // /// </summary>
    // /// <param name="services"></param>
    // /// <returns></returns>
    // public static IServiceCollection UseAutoMapper(this IServiceCollection services)
    // {
    //     services.AddAutoMapper();
    //     return services;
    // }

    /// <summary>
    /// 添加MvcOptions 默认配置
    /// <code>
    ///   mvcOptions.ModelValidatorProviders.Add(new BPAModelValidatorProvider());
    ///   mvcOptions.MaxModelValidationErrors = 20;
    ///   过滤器:RenderBaseResultFilter
    /// </code>
    /// </summary>
    /// <param name="options"></param>
    /// <param name="isGeteway">是否为网关层</param>
    /// <returns></returns>
    public static MvcOptions AddDefaultMvcOptions(this MvcOptions options, bool isGeteway = false)
    {
        ExceptionFilter.IsGeteway = isGeteway;
        
        options.Filters.Insert(0, new BaseResultFilter());
        options.Filters.Add<ExceptionFilter>();
        options.ModelValidatorProviders.Add(new BPAModelValidatorProvider(isGeteway));
        options.MaxModelValidationErrors = 20;
        
        return options;
    }

    /// <summary>
    /// 添加 ApiBehaviorOptions 默认配置,
    /// <code>配置 InvalidModelStateResponseFactory </code>
    /// </summary>
    /// <param name="options"></param>
    /// <returns></returns>
    public static ApiBehaviorOptions AddDefaultBehaviorOptions(this ApiBehaviorOptions options)
    {
        options.SuppressModelStateInvalidFilter = false;
        options.InvalidModelStateResponseFactory = context =>
        {
            var error = context.ModelState
                .FirstOrDefault(s =>
                    s.Value.ValidationState == Microsoft.AspNetCore.Mvc.ModelBinding.ModelValidationState.Invalid &&
                    s.Value.Errors.FirstOrDefault() != null).Value?.Errors.FirstOrDefault();

            var logger = context.HttpContext.RequestServices.GetService<ILogger<HttpContext>>();

            logger.LogError(error?.Exception,
                $"参数错误:{error?.ErrorMessage} \n【reqUrl】:\n\t{context.HttpContext.Request.GetRequestUrl()}\n【reqArgs】:\n\t{context.HttpContext.Request.ReadBodyAsync().Result}");

            var errorMessage = error?.ErrorMessage;

            if (error?.Exception != null)
            {
                errorMessage = null;
            }

            var result = new BaseResult(ConstLangPackge.ParamInvalid, errorMessage, errorMessage)
            {
                TrackId = context.HttpContext.TraceIdentifier
            };
            result.Render(context.HttpContext.Request.GetLanguage());
            return new JsonResult(result);
        };
        return options;
    }

    private static void InitJsonSerializerSettings(JsonSerializerSettings serializerSettings)
    {
        serializerSettings.Converters.Add(new StringEnumConverter(new CamelCaseNamingStrategy(), true));
        //设置json日期格式
        serializerSettings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
        serializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
        serializerSettings.Converters.Add(new JsUlongConverter());
        serializerSettings.Converters.Add(new JsDateTimeConverter());
    }

    /// <summary>
    /// 启用全局拦截
    /// </summary>
    /// <param name="builder"></param>
    /// <param name="isGeteway">是否网关</param>
    /// <returns></returns>
    public static IApplicationBuilder UseGlobalErrorHandler(this IApplicationBuilder builder, bool isGeteway = false)
    {
        builder.Use(async (context, next) =>
        {
            context.Request.EnableBuffering();
            if (context.Request.Headers.TryGetValue(RequestIdHeadName, out StringValues requestID) && requestID.Count > 0)
            {
                context.TraceIdentifier = requestID;
            }
            else if (context.Request.Headers.TryGetValue(TraceIdHeadName, out StringValues traceId) && traceId.Count > 0)
            {
                context.TraceIdentifier = traceId;
            }
            else
            {
                if (context.TraceIdentifier.IsEmpty())
                {
                    context.TraceIdentifier = Guid.NewGuid().ToString("n");
                }
                else
                {
                    context.TraceIdentifier = context.TraceIdentifier.Replace(":", string.Empty).Replace(".", "d");
                }
            }
#if DEBUG
            System.Diagnostics.Debug.WriteLine($"{TraceIdHeadName}:{context.TraceIdentifier}");
            Console.WriteLine($"{TraceIdHeadName}:{context.TraceIdentifier}");
#endif
            context.Response.Headers.Add(TraceIdHeadName, context.TraceIdentifier);
            await next.Invoke();
        });

        builder.UseExceptionHandler(builder => builder.Run(async context => await ErrorEvent(context, isGeteway)));
        return builder;
    }

    /// <summary>
    /// 全局异常拦截
    /// </summary>
    /// <param name="context"></param>
    /// <param name="isGeteway"></param>
    /// <returns></returns>
    private static Task ErrorEvent(HttpContext context, bool isGeteway = false)
    {
        var error = context.Features.Get<IExceptionHandlerFeature>()?.Error;

        if (error == null)
            return Task.CompletedTask;

        var logger = context.RequestServices.GetService<ILogger<HttpContext>>();
        logger?.LogError(error, "未处理异常");

        var result = BaseResult.Build(ConstLangPackge.Error, "请稍后重试", "please try again later").Render();
        result.TrackId = context.TraceIdentifier;

        context.Response.StatusCode = isGeteway ? 200 : 500;
        context.Response.ContentType = "application/json";

        return context.Response.WriteAsync(JsonConvert.SerializeObject(result), Encoding.UTF8);
    }
}